/*
 * This code tests the tdfxy rototation correction code.
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#define DAS2R 4.8481368110953599358991410235794797595635330237270e-6
#define DD2R 0.017453292519943295769236907684886127134428718885417
#define gmax(A,B) ((A)>(B)?(A):(B))
#define dsign(A,B) ((B)<0.0?-(A):(A))

double TdfHfxTerm(double ha, double dec, double z);
void slaDcs2c ( double a, double b, double v[3] );


static char *rcsId="@(#) $Id: ACMM:2dFutil/rotation_test.c,v 5.2 10-Sep-2013 09:12:12+10 tjf $";
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);

int main(int argc, char *argv[])
{
    double NP=0.24  * DAS2R;
    double CH=-22.3 * DAS2R;
    double HF=-4    * DAS2R;
    double ZGT=0    * DAS2R;

    double tf;
    double r;
    double sda;
    double cda;
    double dp;
   
    double x1=0;
    double y1=0;
    double z1=0;

    double ha = 0;
    double dec = 0;
    double hfx;

    double vec1[3];

    if (argc < 4)
    {
        fprintf(stderr,"Usage  HA DEC CH\n");
        exit(1);
    }
    
    ha = atof(argv[1]);
    dec = atof(argv[2]);
    CH = atof(argv[3]);
    

    printf("Using HA=%.2lf degrees, Dec=%.2lf Degrees, CH = %.2lf arc-seconds\n", 
           ha, dec, CH);
    
    CH  *= DAS2R;
    ha  *= DD2R;
    dec *= DD2R;

   slaDcs2c(ha,dec,vec1);
   x1 = vec1[0];
   y1 = vec1[1];
   z1 = vec1[2];
    
   
   hfx = TdfHfxTerm(ha,dec,z1);


   /* Some magic number (tube flexure) * AS to radians. */

   tf = -18.72*DAS2R;  

   /* Get the radius of of the object.  */

   r = gmax(sqrt(x1*x1+y1*y1),1e-10);


   sda = (CH- (HF +0.86*tf) * sin(ha) + hfx + (NP) *z1)/r  + ZGT;
   if (fabs(sda) > 0.999999) sda = dsign(0.999999,sda);
   cda = sqrt(1.0-sda*sda);
   dp = (-atan2(sda,cda)-(NP)*r)*z1;


   printf("DP is %.3f degrees\n", dp/DD2R);
   return 0;
}

double TdfHfxTerm(double ha, double dec, double z)

{
    double h2,h3,h5;
    double d2;
    double sh,s2h,s3h,s4h;
    double efx;

    h2 = ha*ha;
    h3 = h2+ha;
    h5 = h3+h2;
    d2 = dec+dec;
    sh = sin(ha);
    s2h = sin(h2);
    s3h = sin(h3);
    s4h = sin(h2+h2);

    efx = +0.219 * sin(h5)
          +0.368 * cos(h5+ha)
          +0.195 * sin(h5+h5+h5)
          +(+1.279*s2h
            -1.718*s4h
            +1.202*cos(h5))*z
          -1.928*s3h*cos(dec)
          +(+3.662*sh
            +0.381*cos(h3))*sin(d2)
          -1.156*s3h*cos(d2)
          +(-0.633*s2h
            +0.723*sh)*sin(d2+d2+dec);

     return DAS2R*efx;
}

