/*
 *+                         t d F a u t o T h e t a . c
  
 * Function name:
      tdFautoTheta
  
 * Function:
 	For a given 2dF button and position, determine theta
  
 * Description:
  	For a given 2dF button and position, determine theta such that
  	there is no twist on the fibre as it exists the button.
       
 * Language:
      C
  
 * Call:
     (double) = tdFautoTheta (pivotx,pivoty,x,y)
  
 * Parameters:   (">" input, "!" modified, "W" workspace, "<" output)
 	(>) pivotx	(long int) Button pivot x position
 	(>) pivoty	(long int) Button pivot y position
 	(>) x		(long int) Button X position.
 	(>) y		(long int) Button Y position.
 	
 * Include files: tdFutil.h
 
 * Returned value:
 	The theata value.
  
 * Support: Tony Farrell, AAO
  
 *-
  
 * History:
      08-Oct-1996 - TJF - Original version, based on tdFpos 
      		interfaceClass::autoTheta by James Wilcox. 
      07-Nov-1996 - TJF - Since this cannot fails, we return the theta value.
  
 */


static char *rcsId="@(#) $Id: ACMM:2dFutil/tdFautoTheta.c,v 5.2 10-Sep-2013 09:12:11+10 tjf $";
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);


#include "tdFutil.h"
#include <math.h>

#define  PI  3.1415926535

extern double tdFautoTheta(long int pivotx, long int pivoty, 
			 long int butX,   long int butY)
{
    double  dx, dy, tmpdx, tmpdy;
    double theta;
    
    /*
     *  Determine theta such that it lies along a straight line between
     *  the fibre-end and the pivot point.
     */
    dx = (double)pivotx - (double)butX;
    dy = (double)pivoty - (double)butY;
    tmpdx = (dx < 0)? -dx: dx;
    tmpdy = (dy < 0)? -dy: dy;
    if (dx>0 && dy==0)
        theta = 3*PI/2;
    else if (dx>0 && dy>0)
        theta = 3*PI/2 + atan((double)tmpdy/(double)tmpdx);
    else if (dx==0 && dy>0)
        theta = 0;
    else if (dx<0 && dy>0)
        theta = PI/2 - atan((double)tmpdy/(double)tmpdx);
    else if (dx<0 && dy==0)
        theta = PI/2;
    else if (dx<0 && dy<0)
        theta = PI/2 + atan((double)tmpdy/(double)tmpdx);
    else if (dx==0 && dy<0)
        theta = PI;
    else /* (dx>0 && dy<0) */
        theta = 3*PI/2 - atan((double)tmpdy/(double)tmpdx);
    return theta;
}
			 
/*
 *+                         t d F a u t o T h e t a 
  
 * Function name:
      tdFautoThetaPos
  
 * Function:
 	For a given 2dF position, determine theta
  
 * Description:
  	For a given 2dF field position, determine theta such that it
  	is appropiate for a button a that position.
       
 * Language:
      C
  
 * Call:
     (double) = tdFautoThetaPos (x,y)
  
 * Parameters:   (">" input, "!" modified, "W" workspace, "<" output)
 	(>) x		(long int) Button X position.
 	(>) y		(long int) Button Y position.
 	
 * Include files: tdFutil.h

 * Returned value:
 	The theata value.

  
 * Support: Tony Farrell, AAO
  
 *-
  
 * History:
      07-Nov-1996 - TJF - Original version
      04-Sep-2004 - TJF - Catch case of x and y both zero - and return 0.
 */
extern double tdFautoThetaPos(long int x,   long int y)
{
    double           tmpdx,tmpdy;
    double theta;

    if ((x == 0)&&(y == 0))
        return 0;

/*
 *  Calculate theta such that is will be correctly align for the point on
 *  the outer radius nearest to where we are.
 *
 */
    tmpdx = (x < 0)? -1*x: x;
    tmpdy = (y < 0)? -1*y: y;
    if (x>0 && y==0)
        theta = 3*PI/2;
    else if (x>0 && y>0)
        theta = 3*PI/2 + atan(tmpdy/tmpdx);
    else if (x==0 && y>0)
        theta = 0;
    else if (x<0 && y>0)
        theta = PI/2 - atan(tmpdy/tmpdx);
    else if (x<0 && y==0)
        theta = PI/2;
    else if (x<0 && y<0)
        theta = PI/2 + atan(tmpdy/tmpdx);
    else if (x==0 && y<0)
        theta = PI;
    else /* (x>0 && y<0) */
        theta = 3*PI/2 - atan(tmpdy/tmpdx);
        
    return theta;
}
			 
