

static char *rcsId="@(#) $Id: ACMM:2dFutil/tdffit.c,v 5.2 10-Sep-2013 09:12:11+10 tjf $";
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);


#include <stdio.h>
#include <stdlib.h>
#include "slalib.h"
#include "Ers.h"
#include "DitsMsgOut.h"
#include "sds.h"
#include "arg.h"
#include "tdfxy.h"
#include "tdfxyfit.h"



void FitProgress(float pc DUNUSED)

{  }


void TdfFit(SdsIdType cal, TdfXyType *xypars, TdfDistType *dist,
            double coeffs[6], StatusType *status)

{
  SdsIdType fid;
  double ra,dec,mjd;    /* Field centre and mjd from input file  */
  double rc,dc;         /* Field centre apparent position  */
  double gmap[21];      /* Mean to apparent parameters  */
  long ndims;
  unsigned long dims[7];
  SdsIdType uid,RaId,DecId,xid,yid;
  int size;
  SdsCodeType code;
  char name[16];
  double *xyp;
  double *xym;
  double *rd;
  char *ex;
  int xi,yi;
  double r,d;
  int i;
  unsigned long actlen;
  double rms2,psd;
  double ps[12];

  int fixlin[6] = { 0, 0, 0, 0, 0, 0 };
  int fixdist[6] = { 1, 1, 1, 1, 0, 0 };

  if (*status != STATUS__OK) return;

/*  Set up initial values for linear and distortion parameters  */

  coeffs[0] = 0.0;
  coeffs[1] = 1.0;
  coeffs[2] = 0.0;
  coeffs[3] = 0.0;
  coeffs[4] = 0.0;
  coeffs[5] = 1.0;

  dist->a = 1.33667137e7;
  dist->b = 1.08945e9;
  dist->c = 6.210e11;
  dist->d = 3.312e14;
  dist->x0 = 0.0;
  dist->y0 = 0.0;

/*  Get Field Centre Details  */

  SdsFind(cal,"fieldData",&fid,status);
  ArgGetd(fid,"cenRa",&ra,status);
  ArgGetd(fid,"cenDec",&dec,status);
  ArgGetd(fid,"configMjd",&mjd,status);
  slaMappa(2000.0,mjd,gmap);
  slaMapqkz(ra,dec,gmap,&rc,&dc);
  if (*status != STATUS__OK)
    {
      ErsRep(0,status,"Error getting Field Centre Data");
      return;
    }

/*  Get Star data  */

  SdsFind(cal,"unallocGuide",&uid,status);
  SdsFind(uid,"ra",&RaId,status);
  SdsFind(uid,"dec",&DecId,status);
  SdsFind(uid,"x",&xid,status);
  SdsFind(uid,"y",&yid,status);
  SdsInfo(RaId,name,&code,&ndims,dims,status);
  size = dims[0];

  xym = (double*) malloc(2*size*sizeof(double));
  xyp = (double*) malloc(2*size*sizeof(double));
  rd = (double*) malloc(2*size*sizeof(double));
  ex = (char*) malloc(size);

  for (i=0;i<size;i++)
    {
      SdsGet(RaId,8,i,&r,&actlen,status);
      SdsGet(DecId,8,i,&d,&actlen,status);
      SdsGet(xid,4,i,&xi,&actlen,status);
      SdsGet(yid,4,i,&yi,&actlen,status);
      xym[2*i] = (double)xi;
      xym[2*i+1] = (double)yi;
      slaMapqkz(r,d,gmap,&rd[2*i],&rd[2*i+1]);
      if (xi == 1000000) ex[i] = 1; else ex[i] = 0;
    }
  if (*status != STATUS__OK)
    {
      ErsRep(0,status,"Error getting Star Data");
      return;
    }

  TdfFullFit(size,rd,xym,ex,rc,dc,xypars,dist,mjd,coeffs,
             fixdist,fixlin,xyp,ps,&rms2,&psd,FitProgress,status);
  if (*status != STATUS__OK)
    {
      ErsRep(0,status,"Error in Fit");
      return;
    }

  printf(" Linear              Distortion \n");
  printf(" a = %12.5f    a = %12.5g \n",coeffs[0],dist->a);
  printf(" b = %12.5f    b = %12.5g \n",coeffs[1],dist->b);
  printf(" c = %12.5f    c = %12.5g \n",coeffs[2],dist->c);
  printf(" d = %12.5f    d = %12.5g \n",coeffs[3],dist->d);
  printf(" e = %12.5f   x0 = %12.5g \n",coeffs[4],dist->x0);
  printf(" f = %12.5f   y0 = %12.5g \n",coeffs[5],dist->y0);
  printf(" \n");  
  printf(" RMS = %10.2f   PSD = %10.2f \n",rms2,psd);

  free(xym);
  free(rd);
  free(xyp);
  free(ex);
  SdsFreeId(yid,status);
  SdsFreeId(xid,status);
  SdsFreeId(RaId,status);
  SdsFreeId(DecId,status);
  SdsFreeId(uid,status);
  SdsFreeId(fid,status);

}

