/*
 * Simple test program to help debug tdFcollisionButBut()
 *
 * Arguments:
 *    Button1 X (microns)
 *    Button1 Y (microns)
 *    Button1 T (radians)
 *
 *    Button2 X (microns)
 *    Button2 Y (microns)
 *    Button2 T (radians)
 *
 * The output is the distance at which a collision occurs.
 *   Range 10 microns to 10000 microns.
 *
 * Button X/Y is validate to be on the 2dF field plate.
 *
 * "@(#) $Id: ACMM:2dFutil/testbutbut.c,v 5.2 10-Sep-2013 09:12:12+10 tjf $"
 */

static const char *rcsId="@(#) $Id: ACMM:2dFutil/testbutbut.c,v 5.2 10-Sep-2013 09:12:12+10 tjf $";
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);

#include "tdFcollision.h"
#include "tdFfieldDims.h"

#include <stdlib.h>
#include <stdio.h>


static void usage(const char *argv0)
{
    fprintf(stderr,"Usage:%s But1X But1Y But1T But2X But2Y But2T\n", argv0);
    exit(1);
}

#define CLEAR_MIN 1
#define CLEAR_MAX 10000

int main(int argc, char *argv[])
{
    int but1X=0;
    int but1Y=0;
    double but1T=0;
    int but2X=0;
    int but2Y=0;
    double but2T=0;

    int clearance=CLEAR_MIN;
    int result;
    if (argc != 7)
    {
        fprintf(stderr, "Wrong number of arguments\n");
        usage(argv[0]);
        return 1;
    }

    but1X = atoi(argv[1]);
    but1Y = atoi(argv[2]);
    but1T = atof(argv[3]);

    but2X = atoi(argv[4]);
    but2Y = atoi(argv[5]);
    but2T = atof(argv[6]);
    
    if ((but1X < -FIELD_RAD)||(but1X > FIELD_RAD))
    {
        fprintf(stderr,"Button 1 X (%d) is outside field plate\n", but1X);
        return 1;
    }
    if ((but1Y < -FIELD_RAD)||(but1Y > FIELD_RAD))
    {
        fprintf(stderr,"Button 1 Y (%d) is outside field plate\n", but1Y);
        return 1;
    }

    if ((but2X < -FIELD_RAD)||(but2X > FIELD_RAD))
    {
        fprintf(stderr,"Button 2 X (%d) is outside field plate\n", but2X);
        return 1;
    }
    if ((but2Y < -FIELD_RAD)||(but2Y > FIELD_RAD))
    {
        fprintf(stderr,"Button 2 Y (%d) is outside field plate\n", but2Y);
        return 1;
    }

    printf("Checking button at %d,%d,%g against button at %d,%d,%g\n",
           but1X,but1Y,but1T,but2X,but2Y,but2T);
    
    while ((result==0)&&(clearance <= CLEAR_MAX))
    {
        result = tdFcollisionButBut(but1X,but1Y,but1T, but2X,but2Y, but2T,
                                    clearance);

        if (result)
        {
            printf("  Clearance is %d microns\n", clearance-1);
            return 0;
        }

        ++clearance;
    }
    printf("  Clearance is at least %d microns\n", clearance);
    return 0; 

}
