/*+           S I X D B U T T O N D I M S

 *  Module name:
      sixdFbuttonDims.h

 *  Function:
      Include file containing the dimensions for the 6dF fibre buttons.

 *  Description:
 
 *  Language:
      C

 *  Support: Tony Farrell, AAO.
             Nike Frampton, AAO.

 *-

 *      @(#) $Id: ACMM:6dFutil/sixdFbuttonDims.h,v 4.3 06-Mar-2013 08:32:34+11 tjf $

 *  History:
      13-Dec-1999  TJF   Original version. 
      24-Feb-2000  TJF   Add FID_CLEAR, remove RCS header items and insert
                         SCCS header item.
      22-Jun-2000  TJF   Change MAX_PIV_ANGLE from 0.52 (29.8 degrees) to
                         0.1658 (9.5 degrees) to avoid crossing parked 
                         positions. Exact figure still to be determined.
      10-Aug-2000  TJF   The fibre extension should be measured from
			 the pivot position, not the park position.
      07-Feb-2001  TJF   Drip devinition of FIELD_RAD, since we are using
	                 FIELD_RAD_INC_PVT we must always be including
	 		 sixdFfieldDims.h anyway.  
                         Set  MAX_PIV_ANGLE to 0.3926 (22.5 degrees).
     08-FEb-2001  TJF    For the moment - don't allow extension past centre.
     13-Feb-2001  TJF    Go back to allowwing fibre extension to 25mm past
	                 center - which is the default value.
     20-Feb-2001  TJF    Set FIB_DIA to 1000 - allowing for guide bundles and
                         springs.  Make FIBRE_CLEAR just POS_ACCURACY.
     08-Sep-2009  TJF    Increase FIBRE_CLEAR from 500 to 800 above POS_ACCURACY, since
                           we have had some instances of  buttons being placed on fibres,
                           due apparently to variations in the fibre path.
                         Increase POS_ACCURACY from 15 to 35.  Old value is pretty
				optimistic.
 

 */

#ifndef __SIXDFBUTTONDIMS_H__
#define __SIXDFBUTTONDIMS_H__

/*
  
             (Crude) Plain view of Button.
  
                  RAD                 TAIL
                   |                   |
                   v                   v
      
                        ---
                      -     - 
                    -        -----------          <-THW
                   -     o             +------  
                    -        -----------
                      -     -
                        ---
                         ^
                         |
                         |
                       Origin - center of microprism.

    The 6dF button is circular but there is a tail on the fibre.

    

 */


/*
 *  Note: all dimensions in microns unless stated.
 */
#define  POS_ACCURACY          35        /* Spec-ed positioner accuracy     */

#define  BUTTON_RAD          2500        /* Button raduis - 2.5 mm          */
#define  BUTTON_TAIL        15000        /* Length of tail                  */
#define  BUTTON_THW           318        /* Tail half width                 */
#define  BUTTON_TD            636        /* Tail width			    */

#define  FIBRE_DIA           1000        /* Diameter of fibre optics  (?)   */

#define  EXTENSION     (FIELD_RAD_INC_PVT+25000)  /* Maximum allowed extension, 
                                            cen + 25mm                        */

#define  BUTTON_CLEAR (100+POS_ACCURACY) /* Minimum allowed button/button 
                                            clearance (no space required for   
                                            gripper)                        */
#define  FIBRE_CLEAR     (POS_ACCURACY)+800 /* Minimum allowed button/fibre
                                            clearance.   */

#define  FID_CLEAR    FIBRE_DIA        /* Minimum allowed fiducial/fibre or
                                           button clearance.                */

#define  MAX_PIV_ANGLE      0.3926      /* +/- max allowed piv/fib angle 
                                            (rads) - determined by Will
                                              Saunders - Feb 2001  */

#endif
