
/*+           S I X D F F I E L D D I M S

 *  Module name:
      sixdFfieldDims.h

 *  Function:
      Include file containing the dimensions for the 6dF field plates.

 *  Description:
 
 *  Language:
      C

 *  Support: Tony Farrell, AAO.
             Niki Frampton, AAO.

 *-

 *     @(#) $Id: ACMM:6dFutil/sixdFfieldDims.h,v 4.3 06-Mar-2013 08:32:34+11 tjf $


 *  History:
      13-Dec-1999  TJF   Original version.  A lot still be be done.  The
                         Field radius should be ok as is the fiduical
                         hole diameter, most of the other
                         still need real values.
      24-Feb-2000  TJF   Remove RCS header items, insert Sccs id.  Add more
                         6dF items.
      03-Apr-2000  TJF   FIELD_RAD_INC_PARK modified to
	                 FIELD_RAD_INC_PVT (currently assuming pivot
	                 point is 20mm outside plate radius, but this
		         needs to be determined mathmatically.
      19-Dec-2000  TJF   Update motor limits in encoder readings based
                         on hardware tests.
      20-Dec-2000  TJF   Add R_GLIDE_ENC and R_NOFLOAT_ENC - approximate
                         values only.
      22-Dec-2000  TJF   Redefine RMAX to be Field radius including pivot
                         point, mut it may need to be reduced slightly from
                         this.
      16-Jan-2001  TJF   Remove GRIP_CLEAR values - not significant for 6dF.
      18-Jan-2001  TJF   Drop THETA_MAX_ENC 1793000 to 1785000 after theta 
			 limit adjustment due to centering of theta over
	                 plate 1.
      29-Jan-2001  TJF   Revamp Z operation enum's - 6dF gripper is more
                         an operation based system rather then a positioner
                         based system.
			 Phi min/max were wrong.  Fix.
      06-FEb-2001 TJF    Adjust FIELD_RAD and  FIELD_RAD_INC_PVT based
                         on implementation of the widgets.  Add PARK_RAD, 
		         which is used to help determine the park positions.
			 Define RMAX to be PARK_RAD+500 instead of pivot 
			 radius.
      08-Feb-2001 TJF    Adjust R_MAX_ENC and R_NOFLOAT_ENC based
                         on experimentation.
      19-Feb-2001 TJF    Adjust R_MAX_ENC, R_NOFLOAT_ENC and  R_GLIDE_ENC 
                         based on  hardware revamp and proper implementation
                         of glide zone.  Add  GRIP_R_ACC_MIN  and
                         GRIP_R_ACC_MAX
      23-Feb-2001 TJF    Glide zone adjusted again.
                         Addjust THETA_REV_ENC.
      30-Mar-2001 TJF    Adjust PLATE_CEN_INV to 510 (measured value is
                         currently 503)
      13-May-2002 WS     R_GLIDE_ENC  116100  from 11810, R_NOFLOAT_ENC 122000 from 123900
      13-May-2002 WS     changed R_MAX_ENC to 128000 from 127850 lost all patience
      13-May-2002 WS     changed R_MAX_ENC to 128100 from 128000 still being triggered
      16-May-2002 WS     changed THETA_MAX_ENC  1785000 ->  1770000 because hitting theta +ve servostar limit
      21-Sep-2002 TJF   Change R_NOFLOAT_ENC to 124500.
      14-Jun-2007 TJF   Change THETA_MIN_ENC from  -303000 to -356879
                        Change THETA_MAX_ENC  from  1770000 to 1739357
                        These we due to a robot rebuild.


*/

#ifndef __SIXDFFIELDDIMS_H__
#define __SIXDFFIELDDIMS_H__

#ifdef __cplusplus
 extern "C" {
#endif


/*
 * Usable field plate radius.  The actual field plate diameter is
 * 322mm.  Buttons are parked at 157.5.  The usable field
 * radius is 152.5
 *
 * The fibre pivot point is at a radius of 164.25, which is determined
 * by the geometry of the widgets.
 */
#define FIELD_RAD        152500 
#define PARK_RAD         157500
#define FIELD_RAD_INC_PVT 164250 /* Field radius including pivot radius     */
#define FID_DIA             900  /* Fiducial hole diameter (microns)        */
#define FID_RAD             450  /* Fiducial hole radius   (microns)        */

#define PLATE_CEN_INV       510  /* Radius of circle at center where we 
                                    cannot place a button due to the gripper 
                                    not being able to read it 
                                    */
     


#define RMIN             -90000  /* R-axis minimum value (microns)           */
#define RMAX             PARK_RAD+1500/* R-axis maximum value (microns)           */
#define R_MIN_ENC        -53000  /* Minimum r encoder value                  */
#define R_MAX_ENC        128100  /* Maximum r encoder value                  */
                                 /* NOTE - gripper task uses R_POS_LIMIT */
                                 /* parameter instead of this macro in   */
                                 /* all significant cases        */

#define R_GLIDE_ENC      116100  /* R greater then this is in glide zone     */
#define R_NOFLOAT_ENC    124500  /* If R is greater then this, then gripper   
                                    should not be in float due to the glide
                                    arrangement                              */

#define XMAX             285000  /* Equivalent X coordinate maximum          */
#define XMIN            -285000  /* Equivalent X coordinate minimum          */
#define YMAX             285000  /* Equivalent Y coordinate maximum          */
#define YMIN            -285000  /* Equivalent Y coordinate minimum          */

#define THETA_REV_ENC    1887197 /* Number of theta encoder units in one 
                                    revolution */
#define THETA_MIN_ENC    -300000  /* Minimum theta encoder value              */
#define THETA_MAX_ENC    1801185  /* Maximum theta encoder value              */
#define MAX_RADIAN_VAL   6.754   /* Maximum theta value                      */

/* 
 * Special values used with Z to arguments to the gripper to indicate
 *  operations 
 */
#define Z_DO_PLACE       2      /* Place a button */
#define Z_DO_PICK        0      /* Pick a button  */
#define Z_SET_FLOAT      1      /* Set to float position */
#define Z_LOWER_TO_PLATE 3      /* Lower gripper to plate without placing */

#define THETAMIN        (-2*PI)  /* THETA-axis minimum value (radians)  */
#define THETAMAX       (2.5*PI)  /* THETA-axis minimum value (radians)  */
#define PHIMIN        (-2.0*PI)  /* PHI-axis (plate rotation) minimum 
                                    (radians)   */
#define PHIMAX         (2.0*PI)  /* PHI-axis (plate rotation) maximum 
                                    (radians)   */


#define GRIP_PARKED     (GRIP_A_R | GRIP_A_THETA)/* Axes required to ... */

#define GRIP_R_PARK           0 /* Gripper X axis park position (microns)    */
#define GRIP_Z_PARK           0 /* Gripper Z axis park position (microns)    */
#define GRIP_JAW_PARK         1 /* Gripper JAW axis park position (microns)  */
#define GRIP_THETA_PARK       0 /* Gripper THETA axis park position (microns)*/

#define JAW_CLOSED            0 /* Approximate position of jaw when closed   */

#define GRIP_R_VEL_MIN         1  /* Minimum r axis feedrate                  */
#define GRIP_R_VEL_MAX       300  /* Maximum r axis feedrate                  */
#define GRIP_R_ACC_MIN         1  /* Minimum r axis acceleration              */
#define GRIP_R_ACC_MAX      4000  /* Minimum r axis acceleration              */
#define THETA_VEL_MIN          1  /* GRIP theta axis minimum feedrate value   */
#define THETA_VEL_MAX        100  /* GRIP theta axis maximum feedrate value   */

#define FP_PIV_DO	13000	/* Pivot distance outside field plate (?) */

#ifdef __cplusplus
}
#endif
#endif

