#ifndef _SIXDFPOS_INC
#define _SIXDFPOS_INC
#ifdef __cplusplus
extern "C" {
#endif

/*
 *+			S I X D F P O S

 * Include File name:
      sixdfpos.h

 * Function:
      Header file for the 6dF position transformation library.

 * Description:

 * Language:
      C


 * Support: Tony Farrell, Ken Russell, AAO

 *-

 *     @(#) $Id: ACMM:6dFutil/sixdfpos.h,v 4.3 06-Mar-2013 08:32:34+11 tjf $

 * History:
     03-Dec-1999 - TJF - Original version
     24-Feb-2000 - TJF - Remove temp error code.
     10-Mar-2000 - TJF - Add extra arguments to support search paths
                         and reporting of which file was opended. 
     17-Mar-2000 - TJF - Add SixdfposQuickAppToObs
     30-Sep-2002 - TJF - Add field center wavelength to CvtInit().
 */

/*
 *  Include files
 */
#include "status.h"
#include "fpil.h"
/* 
 * Constant definitions
 */
/*
 * Macro function definitions
 */

/*
 * type definitions
 */

/*
 * Exported functions.
 */
extern void SixdfposTransAccess (
    void * clientData,
    const char *dirOverride,
    const char *dirFallback,
    FpilRptRoutineType RptRoutine,
    void * RptArg,
    unsigned int field,
    FpilModelType **model,
    StatusType *status);

extern void SixdfposCvtInit (
    FpilModelType *model,
    double mjd,
    double dut,
    double temp,
    double pres,
    double humid,
    double cenWave,
    double obsWave,
    const double * params,
    StatusType *status);

extern void SixdfposRd2Xy (
    const FpilModelType *model,
    double cra,
    double cdec,
    double ra,
    double dec,
    double mjd,
    double *x,
    double *y,
    StatusType *status);

void SixdfposXy2Pos (
        const FpilModelType *model,
        double x,
        double y,
        double *xp,
        double *yp,
        StatusType *status);

extern void SixdfposXy2Rd (
    const FpilModelType *model,
    double cra,
    double cdec,
    double x,
    double y,
    double mjd,
    double *ra,
    double *dec,
    StatusType *status);

extern void SixdfposPos2Xy (
    const FpilModelType *model,
    double xp,
    double yp,
    double *x,
    double *y,
    StatusType *status);

extern void SixdfposFree (
    const FpilModelType *model,
    StatusType *status);

extern void SixdfposQuickAppToObs  (
    const FpilModelType *model,
    double ra,
    double dec,
    double *aob,
    double *zob,
    double *hob,
    double *dob,
    double *rob,
    StatusType *status);


/*
 * To be used only by FPI Full implementation.
 */
extern void Sixdfpos___CloneModel(
    const FpilModelType * modelSource,
    FpilModelType ** modelClone,
    StatusType *status);

#ifdef __cplusplus
}
#endif
#endif


