/*
 *                       A l l o c  S u b  S y s t e m . h
 *
 *  Function:
 *     Main include file for the allocation sub-system.
 *
 *  History:
 *     22nd Jul 2005. Original version. KS/AAO.
 *     23rd Jul 2005. Added PivotOfTarget() and TargetOfPivot()
 *     which use auxiliary arrays AllocPivots and AllocTargets
 *     respectively. The auxiliary arrays are located in 
 *     CONFIG_ALLOC_Global and are initialised in AllocationInit().
 *     -Swapped parameter order of AllocationIndex().
 *     -Changed #include "configureMethodMatrix.h" 
 *           to #include "ConfigureMethodMatrix.h"
 *     -Added check if the target or pivot to be allocated in IsAllocAble()
 *     is already allocated in IsAllocable().
 *     	BM/MQ_AAO.
 *     24th Jul 2005. Added AllocationGlobalRelease(). KS/AAO.
 *     28th Jul 2005. Added the generic fibre swapping routine Swap(). BM/MQ_AAO.
 *     16th May 2006. Added MaxAllocations to AllocationInit. KS. 
 *     15th Jun 2006. MaxAllocations is now a percentage. KS. 
 *      1st Nov 2006. Added AllocationFinish(). KS/AAO. 
 *      3rd Nov 2006. Added Preferences to AllocationInit() call. KS/AAO.
 *     		      
 *
 * "@(#) $Id: ACMM:aaoConfigure/src/AllocSubSystem.h,v 12.7 04-Nov-2013 11:28:49+11 tjf $"
 */
 
#ifndef __AllocSubSystem__

#define __AllocSubSystem__

/*  DRAMA include files, only status.h needed */

#include "status.h"

#include "tdFconfig.h"
#include "tdFconfigMethod.h"
#include "ConfigMethodMatrix.h"

typedef struct AllocationPrefs {
   char* MatrixFilename;              /* Points to name of file to save data */
   int MaxThreads;                    /* Maximum no. of background threads */
   int OnTheFly;                      /* True for 'on-the-fly' calculations */
} AllocationPrefs;

void AllocationGlobalInit (void);

void AllocationGlobalRelease (void);

void AllocationInit (
   CONFIG_METHOD_MatrixElement
                              **Matrix,/* (>) Matrix of possible allocations */
   const CONFIG_MethodInfo *MethodInfo,/* (>) Method details */
   int MaxAllocPercent,                /* (>) Allocation limit. 0 => no limit.*/
   CONFIG_ReportFunction
            ReportFunction,            /* (>) Feedback routine to call */
   void *FunctionArg,                  /* (>) Caller-supplied argument */
   AllocationPrefs* Preferences,		   /* (>) Preferences to use. */
   StatusType *Status);                /* (!) Inherited status */

void AllocationFinish (
   StatusType* Status);
   
int AllocationIndex (
   int IPivot,
   int ITarget);
   
int AllocateIndex (                    /*  Returns true if index handled OK */
   int Index);                         /*  Index of allocation being made */

int DeallocateIndex (                  /*  Returns true if index handled OK */
   int Index);                         /*  Index of deallocation being made */
   
int AllocEntries(void);

int AllocEntryDetails(
   int Number,                 /* The number of the allocation in question */
   int* Index,                 /* The allocation index for the allocation */
   int* Pivot,                 /* The pivot number - from zero up */
   int* Target,                /* The target number - from zero up */
   double* Theta);             /* The theta value for this allocation */

int IsAllocAble (             /* Returns true if this allocation is valid */
   int Index);                /* Index of the allocation to be tested */

int PivotOfTarget(int ITarget); /*Returns Pivot index associated with ITarget
				  if ITarget is allocated, -1 otherwise */
int TargetOfPivot(int IPivot); /*Returns Target index associated with IPivot
				  if IPivot is allocated, -1 otherwise */
int Swap(int PivotA,int TargetA,int PivotB,int TargetB); /*Generic Swapping Routine*/ 
		  
#endif
