/*
 *                   C o n f i g  M e t h o d  M a t r i x . h
 *
 *  This file contains the definitions used by the template method-specific
 *  code that are also used by the allocation sub-system code.
 *
 *  Really, all this matrix stuff should be moved into the allocation
 *  sub-system, but for the moment it's been put into this file so it
 *  can be included by both the method-specific code and the allocation
 *  sub-system code.
 *
 *
 * "@(#) $Id: ACMM:aaoConfigure/src/ConfigMethodMatrix.h,v 12.7 04-Nov-2013 11:28:50+11 tjf $"
 */

#ifndef __ConfigMethodMatrix__

#define __ConfigMethodMatrix__

/*  These are the codes used for the PathStatus flag in the matrix. These
 *  indicate either that a given path is possible, or the reasons why it
 *  isn't. Originally, all elements are set either to POSSIBLE, WON'T_REACH
 *  BAD_ALPHA, BROKEN, PARK_CLASH or INCOMPATIBLE. Nothing can alter these
 *  conditions, which are physical. As the program progresses, fibres get 
 *  allocated and some of the POSSIBLEs change into IN_USE or ALLOCATED or 
 *  COLLISION, and may change back again if fibres are deallocated.
 *  OUTSIDE means the targets too far out and would be badly vignetted...
 *  SCREWED means it would fall on one of the plate screws, which 
 *  would be very bad news!  IGNORE means no attempt should be made to
 *  allocate the pivot. DISABLED means that this is an allocation that is
 *  possible, but is not being allowed at the moment (for example, to
 *  prevent any sky target allocations).
 */

#define CONFIG_METHOD_POSSIBLE 0
#define CONFIG_METHOD_ALLOCATED 1
#define CONFIG_METHOD_WONT_REACH 2
#define CONFIG_METHOD_BROKEN 3
#define CONFIG_METHOD_FIBRE_IN_USE 4
#define CONFIG_METHOD_INCOMPATIBLE 5
#define CONFIG_METHOD_BAD_ALPHA 6
#define CONFIG_METHOD_COLLISION 7
#define CONFIG_METHOD_TARGET_IN_USE 8
#define CONFIG_METHOD_OUTSIDE 9
#define CONFIG_METHOD_SCREWED 10
#define CONFIG_METHOD_IGNORE  11
#define CONFIG_METHOD_PARK_CLASH 12
#define CONFIG_METHOD_DISABLED 13

/*  The template allocation code uses a matrix to keep track of which
 *  allocations are possible and which are not. At the start of the algorithm,
 *  its elements are set purely on the basis of physical constraints - can the
 *  fibre from this pivot reach that target, given the various restrictions
 *  on bend angle, fibre length, screw positions, etc, and whether the
 *  target and fibre are compatible. As the allocation algorithm runs and
 *  allocations are made, this matrix is updated to reflect allocations that
 *  have become invalid becuase they would collide with allocations already
 *  made. The matrix is a 2-D array of elements of type 
 *  CONFIG_METHOD_MatrixElement, defined here. The most important item
 *  is PathStatus, which shows whether the allocation is possible or not
 *  - it always has one of the values defined as CONFIG_METHOD_POSSIBLE
 *  through to CONFIG_METHOD_DISABLED. The Theta value is included 
 *  because it is used a lot in the code, and it is easier to calculate
 *  all the possible theta values right at the start of the allocation than
 *  to do so on the fly as the code runs. The CollidingPivot item makes it
 *  easier to handle the case when an allocation is cancelled - if a 
 *  pivot is no longer allocated, all allocations that show it as a colliding
 *  pivot may now have becoe possible again.
 */
 
typedef struct CONFIG_METHOD_MatrixElement {
   unsigned char PathStatus;     /* Flags state of the possible path */
   int CollidingPivot;           /* A fibre that collides with the path */
   double Theta;                 /* The theta value for the button */
} CONFIG_METHOD_MatrixElement;

#endif
