/*
 *         T c l _ H a s h  T a b l e   -    E m u l a t o r
 *
 *  Function:
 *      Include file for the Tcl hash table emulation routines for configure.
 *
 *  Description:
 *      To allow the standalone 'test harness' code used to exercise
 *      the method code used by 'configure' - which we prefer to link
 *      without the Tcl libraries - this file provides an cheap and
 *      cheerful emulation of the Tcl hash table routines used by
 *      'configure'. These don't actually use a hash table at all, they
 *      just do a linear search - for the test harness, this shouldn't
 *      matter.
 *
 *  Author:
 *      Keith Shortridge, AAO.
 *
 * "@(#) $Id: ACMM:aaoConfigure/src/HashEmulator.h,v 12.7 04-Nov-2013 11:28:50+11 tjf $"
 *
 *  History:
 *      24th Feb 2005. Original version, KS.
 */

#define TCL_EM_NAME_SIZE 64
#define TCL_EM_MAX_ENTRIES 2048

#define TCL_STRING_KEYS 1

/*  The Hash table exists as a fixed size structure that contains a
 *  number of string and value pairs. To add some efficiency, we keep
 *  track of the highest valued entry so far, so we only look at ones
 *  that have been created when we search.
 */
 
typedef struct Tcl_HashEntry {
   char Name[TCL_EM_NAME_SIZE];
   int Value;
} Tcl_HashEntry;

typedef struct Tcl_HashTableInst {
   Tcl_HashEntry EntryList[TCL_EM_MAX_ENTRIES];
   int HighestEntry;
} Tcl_HashTableInstance;

/*  The definition of Tcl_HashTable, as a pointer to an actual hash table,
 *  makes for some awkward indirections in the code in places, but it
 *  wasn't my choice.
 */
 
typedef Tcl_HashTableInstance* Tcl_HashTable;

/*  Prototypes for the emulated routines */

void Tcl_InitHashTable (
   Tcl_HashTable* HashTable,
   int TypeCode);
   
void Tcl_DeleteHashTable (
   Tcl_HashTable* HashTable);

Tcl_HashEntry* Tcl_CreateHashEntry (
   Tcl_HashTable* HashTable,
   char* Name,
   int* New);

void Tcl_SetHashValue (
   Tcl_HashEntry* Entry,
   int Value);
   
Tcl_HashEntry* Tcl_FindHashEntry (
   Tcl_HashTable* HashTable,
   char* Name);
   
int Tcl_GetHashValue (
   Tcl_HashEntry* Entry);
 
/*  And one that's useful for diagnostics. */
  
void TclEmListHashTable (        
   Tcl_HashTable* HashTable);



