/*
 * Generate a fibre X/Y position list for 2dF with 8 line of symmetry at regular
 * changes in angle around the field and 50 points across each line (+ origin)
 *
 * The output can be feed to xy2fld to generate a field.  E.g.
 *
 *      ./gen_8by | ./xy2fld 2df - > a.fld
 *
 * Note - not part of normal configure bbuild. 
 *
 * To bulid this program:
 *
 *     make CFLAGS="-W -Wall -ansi -g" gen_8by
 *
 * Author: Tony Farrell:
 *
 * History:
 *     02-Sep-2011 - Original Version.
 */

static const char *rcsId="@(#) $Id: ACMM:aaoConfigure/src/gen_8by.c,v 12.7 04-Nov-2013 11:28:47+11 tjf $";
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "../../2dFutil/tdFfieldDims.h"

static void GenerateHeader(void);

extern int main()
{
#define NUM_ANGLES 8
#define NUM_POINTS 50
    double angles[NUM_ANGLES] = { 0, 22.5, 45, 67.5, 90, 112.5, 135, 157.5 };
    int skip_on_min[NUM_ANGLES] = { 0, 1, 0, 1, 0, 1, 0, 1 }; /* Skip on min ciricle */

    int angleNum = 0;
    int offset = FIELD_RAD/(NUM_POINTS/2);
    int maxval = offset*NUM_POINTS/2;
    
    GenerateHeader();
    for (angleNum = 0; angleNum < NUM_ANGLES; ++angleNum)
    {
        int val = -1 * maxval;
        double angle = angles[angleNum]*M_PI/180.0;  /* Convert to radians */
        double c_angle = cos(angle);
        double s_angle = sin(angle);
        int n = 0;
        for (val = (-1*maxval) ; val <= maxval ; val+=offset)
        {
            ++n;

            if ((labs(val) <= offset)&&(skip_on_min[angleNum]))
                continue;
            long x = (long)(round(c_angle*val));
            long y = (long)(round(s_angle*val));
            if ((x == 0)&&(y == 0))
                continue;
            printf("O_%d_%d %ld %ld\n", angleNum, n,x, y);
            
        }
    }
    return 0;
}

static void GenerateHeader(void)
{
    printf("LABEL 2dF Cross Pattern\n");
    printf("* Please change UT date and CENTRE position as required\n");
    printf("*  and then run xy2fld. \n");
    printf("UTDATE 2010 01 07.33\n");
    printf("CENTRE 00 23 37.20  -31 28 12.00\n");
    printf("EQUINOX J2000.0\n");
    printf("* end of required header info\n");
    printf("*\n");
    printf("Origin 0 0\n");

}
