#ifndef _FPCOLMDLCALCENTGAUSS_H_
#define _FPCOLMDLCALCENTGAUSS_H_
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/include/fpcolMdlCalCentGauss.h,v 1.63 07-Oct-2008 11:10:30+10 tjf $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* tfarrell  01/01/01  Created from AAO's Gcam.h module.
*/

/*
 *  Include file for Generic Cam module
 */


typedef int StatusType;

#define GCAM__OK			0
#define GCAM__MALLOCERR		1
#define GCAM__CENT_FITERR	2


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *
 * The default limit for centroiding about the center supplied by the user 
 *  to the CENTROID command.
 */
#define GCAM_C_CENTROID_DEF_LIMIT 20

typedef struct {
	    long int MaxX;	/* Maximum X value			     */
	    long int MaxY;	/* Maximum Y value			     */
	    long int Xoffset;	/* The X offset from the start of the window */
	    long int Yoffset;	/* The Y offset from the start of the window */
	    long int Xdim;	/* The X dimension of the window	     */
	    long int Ydim;	/* The Y dimension of the window	     */
	    int  PixelSize;	/* Bytes per pixel (1,2,4)		     */
	    } GCamWindowType;


typedef enum { GCAM_SHUT=0, GCAM_OPEN=1 } GCamShutterType;
typedef enum { GCAM_EXPOSE, GCAM_READOUT, GCAM_GRAB, GCAM_IMAGE, GCAM_IMAGECMD}
			 GCamPrivateType;
typedef enum { GCAM_STATE_INT, GCAM_STATE_HOLDING, GCAM_STATE_READY_READ,
	       GCAM_STATE_READING, GCAM_STATE_READOUT,
	       GCAM_STATE_CANCELED, GCAM_STATE_ABORTED } GCamStateType;

typedef void (* GCamExposeRoutineType)(
		    int CamNumber,
		    GCamWindowType *window,
		    float exposure_time,
		    float update_time,
		    GCamShutterType shutter,
		    GCamPrivateType private,		    
		    void *client_data,
		    StatusType *status);

typedef void (* GCamReadoutRoutineType)(
		    int CamNumber,
		    GCamWindowType *window,
		    unsigned long size,
		    float update_time,
		    GCamPrivateType private,
		    void *client_data,
		    void *data,
		    StatusType *status);

typedef void (* GCamKickRoutineType)(
		    int immediate,
		    int CamNumber,
		    void *client_data,
		    StatusType *status);

		    
typedef void (* GCamImageRoutineType)(
		    void *client_data,
		    void *data,
		    GCamWindowType *window,
		    StatusType *status);

typedef void (* GCamSelectRoutineType)(
		    	char *string,
			void *client_data,
			long int  *value,
			StatusType *status);

typedef void (* GCamCentroidRoutineType)(
			GCamWindowType *Window,
			  void  *data,
			  int bias,
			  double *results,
			  StatusType *status);

typedef void (* GCamCentNotifyRoutineType)(
			void *client_data,
			double xvalue, double yvalue,
			double xfull,  double yfull,
			double fwhm, StatusType *status);

extern void GCamInitialise (const GCamWindowType *window, 
				StatusType *status);
extern void GCamShutdown (StatusType *status);

/*
extern void GCamActivate (
			const SdsIdType parsysid, 
			const int pixelSize,
			GCamExposeRoutineType expose,
			GCamKickRoutineType exposeKick,
			void * expose_data,
			GCamReadoutRoutineType readout,
			GCamKickRoutineType readoutKick,
			void * readout_data,
			GCamSelectRoutineType select,
			void * SelectData,
			StatusType *status);
*/
extern void GCamDeActivate (StatusType *status);

extern void GCamCentroid (GCamWindowType *Window,
			  void  *data,
			  int bias,
			  double *results,
			  StatusType *status);
extern void GCamCentroidSharp (GCamWindowType *Window,
			  void  *data,
			  int bias,
			  double *results,
			  StatusType *status);
extern void GCamCentroidMed (GCamWindowType *Window,
			  void  *data,
			  int bias,
			  double *results,
			  StatusType *status);

extern void GCamCentroidGauss (GCamWindowType *Window,
                          void  *data,
                          int bias,
                          double *results,
                          StatusType *status);
extern void GCamCentroid2dGauss (GCamWindowType *Window,
                          void  *data,
                          int bias,
                          double *results,
                          StatusType *status);



extern void GCamExposing(GCamPrivateType private,
			      float exposureTime, float exposedTime,
			      float elapsedTime,  
			      GCamShutterType shutterState,
			      GCamStateType state, 
			      StatusType *status);

extern void GCamReading(GCamPrivateType private,
			      GCamStateType state,
			      StatusType *status);

extern void GCamImage (
			    int CamNumber,
			    GCamWindowType * window , 
			    float exposureTime,
			    float updateTime, GCamShutterType shutter,
			    GCamImageRoutineType handler,
			    void *client_data,
			    StatusType *status);

extern void GCamWindowCmd(StatusType *status);
extern void GCamGetWindow(GCamWindowType *window, StatusType *status);
extern void GCamSetWindow(const GCamWindowType *NewWindow,  
				StatusType *status);
extern void GCamReport (StatusType *status);
extern void GCamPutCentRoutine(GCamCentroidRoutineType CentRoutine, 
				  StatusType *status);
extern void GCamCentMeth (StatusType *status);

extern void GCamPutCentNotifyRoutine(
		GCamCentNotifyRoutineType NotifyRoutine, 
		void * clientData,
		GCamCentNotifyRoutineType * OldNotifyRoutine, 
		void * * OldClientData,
		StatusType *status);

#endif
