#ifndef _FPCOLMDLCALCENTROID_H_
#define _FPCOLMDLCALCENTROID_H_

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/include/fpcolMdlCalCentroid.h,v 1.63 07-Oct-2008 11:10:30+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* ssmedley  07/09/01  created
*
*/

/************************************************************************
*   NAME
*	centroid2dGauss centroid centroidGauss kgGaussFit -
*	Miscellaneous centroiding routines for FLAMES/OzPoz
*	Calibration Software.
*
*   SYNOPSIS
*       #include "fpcolMdlCalCentroid.h"
*
*	void centroid (
*		double		*grid,
*		const int	nRows,
*		const int	nCols,
*		double		*pXCentroid,
*		double		*pYCentroid);
*
*	int centroid2dGauss (
*		double		*grid,
*		int		nRows,
*		int		nCols,
*		double		*pX,
*		double		*pY,
*		double		*pFWHM);
*
*	int centroidGauss (
*		double		*grid,
*		int		nRows,
*		int		nCols,
*		double		*pX,
*		double		*pY,
*		double		*pFWHM);
*
*	int kgGaussFit (
*               int             fibre,
*		double		*grid,
*		int		nRows,
*		int		nCols,
*		double		*pX,
*		double		*pY,
*		double		*pFWHM);
*
*   DESCRIPTION
*
*	This library provides several centroiding routines for use with the
*	OzPoz calibration s/w.
*
*	centroid() is the most basic routine which finds the point (x,y) where
*	half the light is above x & half the light below x. Similarly for y.
*
*	centroid2dGauss() tries to find the centroid by fitting the data to a
*	2-dimensional Gaussian. As I (ss) did not write this s/w I know no more
*	about it. Check the implementation for more details.
*
*	centroidGauss() comes from a similar source to centroid2dGauss(). This
*	routine, however, collapses data in x & y & performs 2 1-dimensional
*	Gaussian fits.
*
*	kgGaussFit() also collapses data in x & y & performs 2 1-dimensional
*	Gaussian fits. This implementation comes from Karl Glazebrook's
*	PDL::Fit::Gaussian CPAN module. (converted to C where appropriate.)
*
*   ENVIRONMENT
*
*   CAUTIONS
*
*	kgGaussFit() is by far the better (more accurate) routine & should
*	be preferred over all others.
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
*------------------------------------------------------------------------
*/




void centroid (
	double		*grid,
	const int	nRows,
	const int	nCols,
	double		*pXCentroid,
	double		*pYCentroid);

int centroid2dGauss (
	double	*grid,
	int		nRows,
	int		nCols,
	double	*pX,
	double	*pY,
	double	*pFWHM);

int centroidGauss (
	double	*grid,
	int		nRows,
	int		nCols,
	double	*pX,
	double	*pY,
	double	*pFWHM);

int kgGaussFit (
        int     fibre,
	double	*grid,
	int		nRows,
	int		nCols,
	double	*pX,
	double	*pY,
	double	*pFWHM);

double gaussian (double x, double mean, double stdDev);

#endif
