#ifndef _FPCOLMDLCALKGGAUSSFIT_H_
#define _FPCOLMDLCALKGGAUSSFIT_H_


/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/include/fpcolMdlCalKgGaussFit.h,v 1.63 07-Oct-2008 11:10:30+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* ssmedley  07/09/01  created
*
*/

/************************************************************************
*   NAME
*	PDLGaussianFit -
*	Perform an (effective) 2d gaussian fit by massaging data
*	appropriately & calling Karl Glazebrook's PDL::Fit::Gaussian
*	routine.
*
*   SYNOPSIS
*       #include "fpcolMdlCalKgGaussFit.h"
*
*	int PDLGaussianFit (
*               const int       fibre,
*		const int       nPoints,
*		const double    *aData,
*		double          *pCentre,
*		double          *pFWHM,
*		double          *pBackground,
*		double          *pPeak,
*		double		bg);
*
*
*   DESCRIPTION
*      This library provides an interface to Karl Glazebrook's
*	PDL::Fit::Gaussian centroiding routine.
*
*	PDLGaussianFit() massages the data passed to it into a format
*	that the PDL::Fit::Gaussian centroiding routine expects & then
*	invokes it, returning the result.
*	This code (hopefully) performs the same functionality as
*	the Perl code that comes with the PDL routine.
*
*	Check the implementation for further details.
*
*   ENVIRONMENT
*
*   CAUTIONS
*
*	The current method of determining background noise is
*	temperamental (at best). It is almost always best to
*	provide your own estimation of the background noise,
*	rather than relying on this routine to do it for you.
*
*   EXAMPLES
*
*   SEE ALSO
*
*	PDL::Fit::Gaussian module at www.cpan.org
*
*   BUGS
*
*------------------------------------------------------------------------
*/




/*
 * If <bgNoise> < 0 then this routine will try to determine the
 * background noise itself. O/wise, the value of <bgNoise> is used.
 */

int PDLGaussianFit (
        const int       fibre,
	const int       nPoints,
	const double    *aData,
	double          *pCentre,
	double          *pFWHM,
	double          *pBackground,
	double          *pPeak,
	double		bg);

#endif
