#ifndef _FPCOLMDLCALLOADOFFSETS_H_
#define _FPCOLMDLCALLOADOFFSETS_H_

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/include/fpcolMdlCalLoadOffsets.h,v 1.63 07-Oct-2008 11:10:30+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* tfarrell  16/10/02  Add removeMeanOffsets() prototype.
* tfarrell  26/08/02  Maximum number of buttons is actually 264.
* tfarrell  22/08/02  Add clearButtonOffset() prototype.
* tfarrell  09/05/02  Major revamp to support wavelength specific offsets.
* ssmedley  07/09/01  created
*
*/

/************************************************************************
*  Include file for fpcolMdlCalLoadOffsets.c
*------------------------------------------------------------------------
*/


#include "status.h" /* StatusType */
#include "fpcolMdlCalLibs.h"

#define MAX_BUTTONS		264
#define MAX_OFFSETS             (MAX_BUTTONS*MAX_WLENS)
#define BO_LOADED		1
#define BO_NOT_LOADED	0

typedef struct
{
  double	ra;
  double	dec;
  int           wIndex;       /* Wavelength index */
  int           button;       /* Button number */
  int		bSet;
}RaDec;

/*
 * One offset per button per wavelength.
 */
typedef struct
{
	RaDec	offset[MAX_OFFSETS];
        int     numOffsets;
	int	bLoaded;
}Offsets;


/*
 * load data from from file <pOffsetFilename> which contains
 * information about each button's offset in RA/DEC.
 * Note that file must have data in format:
 * BUTTON_NO RA DEC
 * unit of RA/DEC is: radians
 */
const Offsets * loadButtonOffsets(
                       const char	*pOffsetFilename,
                       FpcalData		*pFpcalData,
                       FpcalCurWaveIdx  *curWaveIndex,
                       int nButtons, 
                       const int *buttons, 
                       StatusType	*pStatus);



int getNumOffsets(void);
/*
 * This function can be used to iterate through all of the
 * offsets that have been set.
 */
const RaDec *getOffset (int index);


/*
 * get the (RA/DEC) offset for the specified button number
 * Note: will return NULL if no offset specified for this button.
 */
const RaDec *getButtonOffset
	(const int	buttonNo,
         const int      waveIndex,
	 StatusType	*pStatus);

const RaDec *getButtonOffset_o(
                               const Offsets *pO,
                               const int	buttonNo,
                               const int      waveIndex,
                               StatusType	*pStatus);
/*
 * return BO_LOADED or BO_NOT_LOADED
 */
int buttonOffsetsStatus (void);


/*
 * set the offset RA/DEC for a specified button.
 */
int setButtonOffset (int buttonNo, double ra, double dec, int waveIndex);


/*
 * Utility function, is the specified button in the specified list.
 */
int buttonFound(                    
                const int nButtons, 
                const int *buttons, 
                const int buttonNo);

/*
 * Clear a button offset.
 */
int clearButtonOffset(int buttonNo, int waveIndex);
int removeMeanOffsets();
#endif
