#ifndef _LOAD_REGIONS_H_
#define _LOAD_REGIONS_H_

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/include/fpcolMdlCalLoadRegions.h,v 1.63 07-Oct-2008 11:10:30+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* ssmedley  07/09/01  created
*
*/

/************************************************************************
*   NAME
*	loadImageLocations xy2index -
*	Load image positions from file.
*
*   SYNOPSIS
*       #include "fpcolMdlCalLoadRegions.h"
*
*	int loadImageLocations (
*		const char	*pPositionsFile,
*		FibreRegion	*aFibreRegions,
*		FibreRegion	*pBackground1,
*		FibreRegion	*pBackground2,
*		int		*pnRegions);
*
*	int xy2index (
*		int		x,
*		int		y,
*		int		nCols);
*
*   DESCRIPTION
*
*	This module is used to read in a set of image positions from disk.
*	The image positions specify the regions of an image that are used to
*	calculate fibre fluxes & background noise. A single region is denoted
*	by 2 coordinates of a rectangle (top-left & bottom-right) which should
*	appear within an UVES slit image.
*
*	The format of the image regions/positions file is:
*	REGION_NO TOP_LEFT_X TOP_LEFT_Y BOTTOM_RIGHT_X BOTTOM_RIGHT_Y\n
*
*	REGION_NO is negative if the region is used for determining background
*	noise. (Note, can only accept 2 background regions).
*
*	loadImageLocations() loads these regions from disk. The 2 background
*	regions are stored in <pBackground1> & <pBackground2>. All other
*	regions are stored in the <aFibreRegions> array & <pnRegions> set to
*	the number of regions this array contains.
*
*	xy2index() is a simple utility function for mapping a pixel (at x,y) into
*	it's position in a 1 dimensional structure. (ie. it allows a 2-dimensional
*	structure to be represented as a 1-dimensional array in C.)
*
*   ENVIRONMENT
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
*------------------------------------------------------------------------
*/




typedef struct
{
	int	x;
	int	y;
}Point;

typedef struct
{
	Point	tl;
	Point	br;
	int		fibreNo;
}FibreRegion;

/*
 * read in a file containing image positions used in calculating
 * fibre fluxes.
 */
int loadImageLocations (
	const char	*pPositionsFile,
	FibreRegion	*aFibreRegions,
	FibreRegion	*pBackground1,
	FibreRegion	*pBackground2,
	int		*pnRegions);

/*
 * Map a pixel (at x,y) into it's position in a 1 dimensional structure.
 * <nCols> is number of columns in image.
 */

int xy2index (
	int		x,
	int		y,
	int		nCols);

#endif
