#ifndef _FPCOLMDLCALMACROS_H_
#define _FPCOLMDLCALMACROS_H_

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/include/fpcolMdlCalMacros.h,v 1.63 07-Oct-2008 11:10:30+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* ssmedley  07/09/01  created
*
*/

/************************************************************************
*   NAME
*	RD2POS POS2RD -
*	Macros to unite 2 commonly consecutively called OzPoz routines.
*
*   SYNOPSIS
*       #include "fpcolMdl.h"
*
*	RD2POS(model,cenRA,cenDEC,ra,dec,date,xPos,yPos,status)
*	POS2RD(model,cenRA,cenDEC,xPos,yPos,date,ra,dec,status)
*
*   DESCRIPTION
*
*	RD2POS() will convert a Ra/Dec coordinate (radians) to
*	a position <xPos>, <yPos> on the field plate, using the
*	specified model, <model>.
*
*	POS2RD() will convert a position <xPos>, <yPos> on the field
*	plate to Ra/Dec coordinates (radians) using the specified
*	model, <model>.
*
*   ENVIRONMENT
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*	fpcolMdlRd2Xy fpcolMdlXy2Pos fpcolMdlPos2Xy fpcolMdlXy2Rd -
*	all in fpcolMdl.h
*
*   BUGS
*
*------------------------------------------------------------------------
*/



#define NUM_ELEMENTS(array) (sizeof(array)/sizeof(array[0]))

#define RD2POS(model,cenRA,cenDEC,ra,dec,date,xPos,yPos,status) \
	{ \
		double _x, _y; \
		fpcolMdlRd2Xy(model, \
				   cenRA, \
				   cenDEC, \
				   ra, \
				   dec, \
				   date, \
				   &_x, \
				   &_y, \
				   status); \
		fpcolMdlXy2Pos(model, \
					_x, \
					_y, \
					xPos, \
					yPos, \
					status); \
	}

#define POS2RD(model,cenRA,cenDEC,xPos,yPos,date,ra,dec,status) \
	{ \
		double	_x, _y; \
		fpcolMdlPos2Xy(model, \
					xPos, \
					yPos, \
					&_x, \
					&_y, \
					status); \
		fpcolMdlXy2Rd(model, \
				   cenRA, \
				   cenDEC, \
				   _x, \
				   _y, \
				   date, \
				   ra, \
				   dec, \
				   status); \
	}

#endif
