/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolDRAMAError.c,v 1.63 07-Oct-2008 11:10:25+10 tjf $"
* 
*
* who       when      what
* --------  --------  ----------------------------------------------
* tfarrell  16/04/02  Added ILLSCALE.
* tfarrell  11/04/02  Added ILLTEMP, ILLPRES, ILLHUMI, ILLWAVE
* tfarrell  20/09/01  created
*/

/************************************************************************
*   NAME
*       fpcolDRAMAErrorToVLTError - Convert a known DRAMA Status code to a 
*       VLT Error code and report.
*
*   SYNOPSIS
*       #include "fpcol.h"
*
*
*       ccsCOMPL_STAT fpistDRAMAErrorToVLTError(
*                             IN  const vltINT32   sdsError,
*                             IN  const char       * location,
*                             OUT ccsERROR         * error)
*   DESCRIPION
*
*  Convert an DRAMA error code to a VLT error and return the VLT error
*
*  We convert all known DRAMA codes (SDS and fpcolMdl) to appropiate VLT fpcol codes. 
*
*  SDS error codes are defined in sds_err.msg, although the values
*  are defined in sds_err.h.  These values must have equivalent
*  entries in the fpcol ERROR module.   We have not bothered with
*  SDS error codes we know we should never get, such as codes related
*  to the SDS compiler or the SDS TCL interface.
*
*  fpcolMdl codes are defined in fpcolMdl_err.msg, althrough the
* values are defined in fpcolMdl_err.h.    These values must have equivalent
*  entries in the fpcol ERROR module. 
*
*  Arguments:
*      dramaError => The SDS error code we are to convert.
*      location => The location of the error, say __FILE__.
*      error    => The VLT error agument.
*
*  Returned Value:
*      FAILURE/SUCCESS.  SUCESS will be returned if the SDS error code
*      is STATUS__OK.*
*      
***
*
*   RETURN VALUES 
*
*   ENVIRONMENT
*      
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*      
*   BUGS   
*
*------------------------------------------------------------------------
*/
/*
 * Define module id for error macros
 */
#define MODULE_ID     "fpcol"
#define lccMODULE_ID  "fpcol"

#define _POSIX_SOURCE 1

static const char *rcsId="@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolDRAMAError.c,v 1.63 07-Oct-2008 11:10:25+10 tjf $"; 
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);

#include "vltPort.h"    /* Must be first header */

/* 
 * Local Headers
 */
#include "fpDefines.h"
#include "fpcol.h"
#include "fpcolErrors.h"
#include "status.h"
#include "sds_err.h"
#include "fpcolMdl_err.h"
#include "err.h"
/*

 */
extern ccsCOMPL_STAT fpcolDRAMAErrorToVLTError(IN  const vltINT32 sdsError,
					       IN  const char *     location,
					       OUT ccsERROR         *error)
{
    /*
     * Largely a simply switch putting the appropiate VLT error code
     * in vltError.  Only for STATUS__OK and unknown codes do we
     * have special cases.
     */
    vltINT32 vltError;
    switch (sdsError)
        {
	/*
	 * First, the SDS codes we may experiences.
	 */
        case SDS__BADID:
            vltError = fpcolERR_SDS_BADID;
            break;
        case SDS__NOMEM:
            vltError = fpcolERR_SDS_NOMEM;
            break;
        case SDS__NOTSTRUCT:
            vltError = fpcolERR_SDS_NOTSTRUCT;
            break;
        case SDS__NOITEM:
            vltError = fpcolERR_SDS_NOITEM;
            break;
        case SDS__NOTPRIM:
            vltError = fpcolERR_SDS_NOTPRIM;
            break;
        case SDS__TOOLONG:
            vltError = fpcolERR_SDS_TOOLONG;
            break;
        case SDS__UNDEFINED:
            vltError = fpcolERR_SDS_UNDEFINED;
            break;
        case SDS__LONGNAME:
            vltError = fpcolERR_SDS_LONGNAME;
            break;
        case SDS__EXTRA:
            vltError = fpcolERR_SDS_EXTRA;
            break;
        case SDS__INVCODE:
            vltError = fpcolERR_SDS_INVCODE;
            break;
        case SDS__INVDIMS:
            vltError = fpcolERR_SDS_INVDIMS;
            break;
        case SDS__NOTARRAY:
            vltError = fpcolERR_SDS_NOTARRAY;
            break;
        case SDS__INDEXERR:
            vltError = fpcolERR_SDS_INDEXERR;
            break;
        case SDS__ILLDEL:
            vltError = fpcolERR_SDS_ILLDEL;
            break;
        case SDS__NOTSDS:
            vltError = fpcolERR_SDS_NOTSDS;
            break;
        case SDS__NOTTOP:
            vltError = fpcolERR_SDS_NOTTOP;
            break;
        case SDS__EXTERN:
            vltError = fpcolERR_SDS_EXTERN;
            break;
        case SDS__NOTARR:
            vltError = fpcolERR_SDS_NOTARR;
            break;
        case SDS__VERSION:
            vltError = fpcolERR_SDS_VERSION;
            break;
        case SDS__FOPEN:
            vltError = fpcolERR_SDS_FOPEN;
            break;
        case SDS__FWRITE:
            vltError = fpcolERR_SDS_FWRITE;
            break;
        case SDS__FREAD:
            vltError = fpcolERR_SDS_FREAD;
            break;
        case SDS__TIMEOUT:
            vltError = fpcolERR_SDS_TIMEOUT;
            break;
	/*
	 * Now the codes from FPCOL.
	 */
        case FPCOL_MDL__DIRLENERR:
            vltError = fpcolERR_DIRLENERR;
            break;
        case FPCOL_MDL__NODISTFILE:
            vltError = fpcolERR_NODISTFILE;
            break;
        case FPCOL_MDL__INVNPARS:
            vltError = fpcolERR_INVNPARS;
            break;
        case FPCOL_MDL__MALLOCERR:
            vltError = fpcolERR_MALLOCERR;
            break;
        case FPCOL_MDL__ILLNAME:
            vltError = fpcolERR_ILLNAME;
            break;
        case FPCOL_MDL__ILLRA:
            vltError = fpcolERR_ILLRA;
            break;
        case FPCOL_MDL__ILLDEC:
            vltError = fpcolERR_ILLDEC;
            break;
        case FPCOL_MDL__SLA_ERR_1:
            vltError = fpcolERR_SLA_ERR_1;
            break;
        case FPCOL_MDL__SLA_ERR_2:
            vltError = fpcolERR_SLA_ERR_2;
            break;
        case FPCOL_MDL__SLA_ERR_3:
            vltError = fpcolERR_SLA_ERR_3;
            break;
        case FPCOL_MDL__SLA_ERR_UNKN:
            vltError = fpcolERR_SLA_ERR_UNKN;
            break;
        case FPCOL_MDL__ILLMJD:
            vltError = fpcolERR_ILLMJD;
            break;

       case FPCOL_MDL__ILLTEMP:
            vltError = fpcolERR_ILLTEMP;
            break;
       case FPCOL_MDL__ILLPRES:
            vltError = fpcolERR_ILLPRES;
            break;
        case FPCOL_MDL__ILLHUMI:
            vltError = fpcolERR_ILLHUMI;
            break;
       case FPCOL_MDL__ILLWAVE:
            vltError = fpcolERR_ILLWAVE;
            break;
       case FPCOL_MDL__ILLSCALE:
            vltError = fpcolERR_ILLSCALE;
            break;

	/*
	 * Status ok.
	 */
        case STATUS__OK:
            return SUCCESS;
	/*
	 * Unknown code.
         */      
        default:
	    errAdd(error, lccMODULE_ID, fpcolERR_DRAMA_UNKNOWN, location, "", sdsError);
	    return FAILURE;

       }

    /*
     * Add the error and return FAILURE.
     */
    errAdd(error, lccMODULE_ID, vltError, location, "");
    return FAILURE;

}


/*___oOo___*/
