#!/bin/sh


#
# E.S.O. - VLT project
#
# "@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolMdlCal.sh,v 1.63 07-Oct-2008 11:10:25+10 tjf $"
#
#
# who       when      what
# --------  --------  ----------------------------------------------
# ssmedley  07/09/01  created
#
#
#
#
#   NAME
#	fpcolMdlCal.sh - Generate an Astrometric Model for the OzPos
#	instrument using simulated data, & verify its accuracy.
#
#   SYNOPSIS
#
#
#   DESCRIPTION
#
#	This script generates a dummy offset file for a specified
#	FITS file & invokes <fpcal> to create an Astrometric Model
#	from this data. The residuals output (of the generated/fitted
#	& true astrometric models) are displayed in the <residuals.tcl>
#	graphical utility so the user can visually check the validity
#	of the fitted astrometric model.
#	
#	The -n command-line option allows the user to specify the
#	amount of noise to generate in the dummy offset file.
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#	A complete overview of the OzPos calibration s/w system
#	(including this driver script) is available in:
#	OzPos Calibration Software Manual. (INS-TRE-AUS-13721-0075)
#
#   BUGS
#
#



echo2 () { echo $* 1>&2 ; }
usage () {
	echo2 "usage: $0 [ options ] <FITSfile>"
	echo2 "\navailable options are:"
	echo2 "-h          help"
	echo2 "-n <noise>  amount of random noise to add to system [0.0]"
}

noise=0.0
while getopts "hn:" a ; do
	case $a in
		h)
			usage
			exit 0
			;;
		n)
			noise=$OPTARG
			;;
		\?)
			usage
			exit 1
			;;
	esac
done

shift `expr $OPTIND - 1`

if [ $# != 1 ] ; then
	echo2 "Specify 1 FITS file"
	usage
	exit 2
fi

echo2 "noise set to $noise"

# generate a dummy offset file with <genOffsets>.
# create/fit Astrometric model with <fpcal>.
# display results graphically with <residuals.tcl>.

offsetFile=offset.cal
residualsFile=residuals.out

genOffsets -n $noise -o $offsetFile $* && \
echo "load $*\noffsets $offsetFile\ncreate\nfit\nquit" | fpcal && \
residuals.tcl $residualsFile
