
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolMdlCalCentRes.c,v 1.63 07-Oct-2008 11:10:26+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* tfarrell  07/05/02  Modify for new offset code (multiple wavelengths)
* ssmedley  07/09/01  created
*
*/

/************************************************************************
*   NAME
*	Centroid Residuals - display the delta Ra/Dec for each fibre b/w
*	2 offset files.
*
*   SYNOPSIS
*       #include "fpcolMdlCentRes.h"
*
*	int delta (const Offsets *pA, const Offsets *pB)
*
*   DESCRIPTION
*
*	This rather simple program loads 2 offset files & displays
*	the difference in Ra/Dec (in arcseconds) for each fibre button.
*
*	It is intended to be used to verify the accuracy of other s/w
*	within the OzPoz calibration system. Namely, genOffsets, gffo
*	& fluxOffsets.
*
*	By:
*	a) creating an initial dummy offset file with <genOffsets>,
*	b) generating flux files from the initial offset file with <gffo>, &
*	c) creating a final offset file soley from the flux files with
*	   <fluxOffsets>
*
*	We can use this program to compare the initial & final offset
*	files. Depending on the amount of noise, the delta should be
*	small.
*
*	Note that there is a Bourne Shell script which automates these
*	steps.
*
*	delta() displays the difference in Ra/Dec for each fibre button.
*
*   ENVIRONMENT
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*	centRes.sh
*
*   BUGS
*
*------------------------------------------------------------------------
*/


static const char *rcsId="@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolMdlCalCentRes.c,v 1.63 07-Oct-2008 11:10:26+10 tjf $";

static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);


#include <stdio.h>
#include <stdlib.h>	/* exit */
#include <assert.h>
#include <math.h>	/* fabs */
#include <slamac.h>	/* DR2AS */

#include "fpcolMdlCalLoadOffsets.h"

void usage (const char *pName)
{
	fprintf(stderr, "usage: %s <offsetFile1> <offsetFile2>\n", pName);
	exit(1);
}

int delta (const Offsets *pA, const Offsets *pB)
{
	int i;

        /*
         * Note here we assume only one wavelength.
         */
	assert(pA != NULL && pB != NULL);
	for (i = 0; i < MAX_BUTTONS; i++)
          {
            StatusType status = 0;
            const RaDec *offsetA = getButtonOffset_o(pA, i, 0, &status);
            const RaDec *offsetB = getButtonOffset_o(pB, i, 0, &status);
            if (offsetA && offsetB)
              {
                double	deltaRA  = offsetA->ra  - offsetB->ra;
                double	deltaDec = offsetA->dec - offsetB->dec;
                
                printf("%3d %18.15f %18.15f arcsec=(%9.6f,%9.6f)\n",
                       i,
                       deltaRA,
                       deltaDec,
                       deltaRA * DR2AS,
                       deltaDec * DR2AS);
              }
            else if ((offsetA)||(offsetB))
              goto badData;
	}
	return 0;

badData:
	fprintf(stderr, "Only 1 offset specified for fibre %d\n", i);
	return -1;
}

int main (int argc, char *argv[])
{
	Offsets		o[2];
	StatusType	status = STATUS__OK;
	int		i;
        FpcalCurWaveIdx curWaveIndex = { 0, 0, 0};

	if (argc != 3)
		usage(argv[0]);

	for (i = 0; i < 2; i++)
	{
          o[i] = *loadButtonOffsets(argv[i+1], 0, &curWaveIndex, 0, 0, &status);
          if (status != STATUS__OK)
            {
              fprintf(stderr, "Can't load offset file <%s>\n", argv[i+1]);
              exit(1);
            }
	}
	if (delta(&(o[0]), &(o[1])) != 0)
		exit(2);

	return 0;
}
