#!/bin/sh

#
# E.S.O. - VLT project
#
# "@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolMdlCalCentRes.sh,v 1.63 07-Oct-2008 11:10:27+10 tjf $"
#
#
# who       when      what
# --------  --------  ----------------------------------------------
# ssmedley  07/09/01  created
#
#
#
#   NAME
#	fpcolMdlCalaCentRes.sh - Verify the integrity of several
#	utility programs within the OzPos calibration s/w system.
#
#   SYNOPSIS
#
#
#   DESCRIPTION
#
#	This program verifies the integrity of several utility
#	programs within the OzPos calibration s/w system by
#	applying an operation to an object & then applying the
#	reverse operation to the resultant. Hopefully, the
#	initial & final objects r the same or similar.
#
#	In this case, the operation is the <gffo> utility which will
#	generate a set of flux files from an offsets file; & the
#	reverse operation is the <fluxOffsets> utility which will
#	generate an offsets file from the set of flux files.
#
#	The residuals are the difference b/w the initial & resultant
#	offsets files. Depending upon the amount of noise embedded
#	in the initial offset file (specified by the -n argument)
#	the residuals may or may not be of a significant magnitude.
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#


echo2 () { echo $* 1>&2 ; }
usage () {
	echo2 "usage: $0 [ options ] <FITSfile>"
	echo2 "\navailable options are:"
	echo2 "-c <centroidRoutine>	gaussfit gaussfit2 kgfit [ centroid ]"
	echo2 "-h"
	echo2 "-n <noise>"
	echo2 "-s <spacing>"
}

noise=0.0
spacing=1.0	# arcseconds
centroidingRoutine=""	# default: centroid
while getopts "c:hn:s:" a ; do
	case $a in
		c)
			centroidingRoutine=-$OPTARG
			;;
		h)
			usage
			exit 0
			;;
		s)
			spacing=$OPTARG
			;;
		n)
			noise=$OPTARG
			;;
		\?)
			usage
			exit 1
			;;
	esac
done

shift `expr $OPTIND - 1`

if [ $# != 1 ] ; then
	echo2 "Specify 1 FITS file"
	usage
	exit 2
fi

echo2 "noise set to $noise"
echo2 "spacing set to $spacing"
echo2 "centroiding routine set to $centroidingRoutine"

# create an initial dummy offset file with <genOffsets>.
# generate flux files from this initial offset file with <gffo>.
# create a final offset file soley from the flux files.
# display the residuals between the initial and final offset files with <centRes>.

iOffset=offset.initial
fOffset=offset.final
fluxFilePrefix=b
fluxFilePostfix=""

genOffsets -n $noise -o $iOffset $* && \
gffo -i $iOffset \
	 -s $spacing \
	 -pre "$fluxFilePrefix" \
	 -post "$fluxFilePostfix" \
	 -v && \
fluxOffsets -o $fOffset \
			$centroidingRoutine \
			${fluxFilePrefix}?${fluxFilePostfix} && \
centRes $iOffset $fOffset
