/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolMdlCalFindExe.c,v 1.63 07-Oct-2008 11:10:26+10 tjf $"
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* tfarrell  26/08/02  Created - stolen from the AAO DRAMA IMP Library.
*/

/************************************************************************
*   NAME
*	fpcolMdlCalFindExe - Find the directory containing the execuable.
*
*   SYNOPSIS
*       #include "fpcolMdlCalLibs.h"
*
*       void fpcolMdlCalFindExe
*               (char *Arg0,  
*                char *Directory,
*                int LDir, 
*                int *Known) 
**
*   DESCRIPTION
*       Given the calling programs argv[0] value to main(), return in Directory the
*       name of the directory containing the executable file.
*
*       This routine comes from the AAO DRAMA IMP Library call  ImpZExeDir(), and is
*       little more then a renaming of it and changes to make it compile without
*       DRAMA include files.
*
*       Most of the work is done by the routine findx(), originally written
*       by Greg Limes (limes@eng.sun.com).  Full details can be found below.
*   ENVIRONMENT
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*	For a complete description of how to use this s/w, refer to
*	the OzPos Calibration Software Manual. (INS-TRE-AUS-13721-0075)
*
*   BUGS
*
*------------------------------------------------------------------------
*/


static const char *rcsId="@(#) $Id: ACMM:ozpozFpcol/ws/src/fpcolMdlCalFindExe.c,v 1.63 07-Oct-2008 11:10:26+10 tjf $";

static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);

#define __EXTENSIONS__    /* Pick up extra include file stuff on solaris */
#define _HPUX_SOURCE      /* Pick up extra include file stuff on HP UX */

#include "fpcolMdlCalLibs.h"
#include <sys/param.h>    /* For MAXPATHLEN, at least under HPUX and solaris */
#include <unistd.h>
#include <stdlib.h>
#define IMP_FALSE 0
#define IMP_TRUE  1
/* -------------------------------------------------------------------------- */

/*                         I m p  Z  E x e  D i r
 *
 *  Routine:
 *     ImpZExeDir
 *
 *  Function:
 *     Gives the name of the directory from which the current program was run.
 *
 *  Description:
 *     In systems where individual tasks are started by loading executable
 *     programs from disk, it is often useful for a program to be able to
 *     locate the directory from which it was loaded. This enables it to
 *     find related files, which are often held in the same directory, 
 *     in a straightforward way that doesn not require the use of logical
 *     names (VMS), environment variables (UNIX), or some other such scheme.
 *
 *     The interface to this routine requires the zeroth argument passed to
 *     the program when it was started up. This is needed by the UNIX 
 *     implementation, although not by the VMS version. It returns the full
 *     pathname or file specification of the directory containing the
 *     program currently being run. This will be formatted such that it
 *     can be prepended to a filename to get a valid full file specification.
 *     (So, for example, under UNIX it will end with a '/', under VMS it will
 *     end with a ']'.) If the system is such that the concept does not make
 *     sense (embedded systems like VxWorks), or where it simply cannot be
 *     implemented), a 'Known' flag argument will be returned set false.
 *    
 *  Implementation details for UNIX:
 *     Most of the work is done by the routine findx(), originally written
 *     by Greg Limes (limes@eng.sun.com) which is appended to this file. It
 *     works by looking through the directories specified by the PATH
 *     environment variable looking for the program name passed as the
 *     zeroth argument (although there's a number of additional wrinkles).
 */


extern void fpcolMdlCalFindExe (
    const char * const Arg0,                      /* Zeroth argument to program */
    char * const Directory,                       /* Name of 'exe' directory */
    const int LDir,                               /* Number of chars in Directory */
    int * const Known)                             /* True if Directory is valid */

/*  The following is my (KS) wrap-up of findx(), producing the UNIX version of
 *  ImpZExeDir(). This is a  routine which a) sets up a call to findx()
 *  to get the name of the 'exe' directory, b) manipulates the result
 *  slightly for aesthetic reasons and to make sure the result ends with a
 *  '/' character, and c) returns it in the nul-terminated character string
 *  Directory. The manipulation of the string returned by findx() is because
 *  the findx() algorithm tends to produce strings such as '/home/aatssb/ks/.'
 *  or '/home/aatssb/ks/work/../scrap/.' which are valid but messy if the
 *  result is ever to be output. (This is a small modification of the Figaro
 *  Fortran-callable utility GEN_EXEDIR, which was canibalised to produce
 *  this code, which is probably over fussy for this IMP application.)
 */

{
   /*  Local variables  */

   int Back;                      /* Number of '/' chars to work back through */
   char Char;                     /* Character in directory name */
   char CurrentDir[MAXPATHLEN];   /* Work space used for current directory */
   int Dots;                      /* Dot count between '/' characters */
   char ExeDir[MAXPATHLEN];       /* Receives the exe directory from findx() */
   int Ichar;                     /* Index into ExeDir */
   int Ioutchar;                  /* Index into returned directory string */
   char LastChar;                 /* Last character written into Directory */

   /*  Functions used  */

   int findx();

   /*  Set up the call to findx() to get the directory name  */

   *Known = IMP_FALSE;
   Ioutchar = 0;
   if (findx(Arg0,CurrentDir,ExeDir,(char *)0,(char *)0,getenv("PATH")) == 0) {

      /*  The call to findx() worked, so we now have a directory string. We
       *  combine the copying of it into the Directory string and the tidying
       *  up. We copy it character by character, keeping a count of the number
       *  of dots after each '/', clearing the count on any ordinary character.
       *  If we come to a new '/' or the end of the string with a positive 
       *  dot count, we skip back by a number of '/' chars equal to the 
       *  number of dots we found.  This has the effect of turning a '/./'
       *  sequence into a '/' and a 'xx/../yy' sequence into 'yy'.
       */

      Char = '\0';
      Ichar = 0;
      Dots = 0;
      do {
         Back = 0;
         LastChar = Char;
         Char = ExeDir[Ichar++];
         Directory[Ioutchar] = Char;
         if (Char == '\0') {
            Back = Dots;
            Ioutchar--;
         } else if (Char == '/') {
            Back = Dots;
            Dots = 0;
         } else if (Char == '.') {
            Dots++;
         } else {
            Dots = 0;
         }
         if (Back) LastChar = '/';
         while (Back) {
            if (Directory[--Ioutchar] == '/') Back--;
            if (Ioutchar <= 0) break;
         }
         if (Ioutchar < LDir) Ioutchar++;
      } while (Char);
      if (LastChar != '/') Directory[Ioutchar++] = '/';
      if (Ioutchar < LDir) {
         Directory[Ioutchar] = '\0';
         *Known = IMP_TRUE;
      }
   }

}
/* ---------------------------------------------------------------------------*/

/*                               F i n d x
 *
 *  The following is the body of the code of the routine findx() as written
 *  by Greg Lines. I wanted to include it to keep this file self-contained,
 *  but I didn't want to dive into it making minor changes and so propagate
 *  a 'slightly' different version if I could help it. I have added a
 *  'static' to the declaration of findx() itself in order to hide the name
 *  and keep it a local routine. Other than that this is the code as supplied.
 *  The original release includes a man page which documents the routine fully,
 *  but I have not included that. Note that the main() routine declared in the
 *  code is not normally compiled.
 *
 *  Having said that in the original version of this routine, I have now made
 *  a more significant modification to the findx() code. I have introduced the
 *  variables dp and dstr, making it copy the individual path elements into
 *  dstr rather than work on them in place - the original code modified the
 *  PATH string it was passed, and so could not be called more than once. I
 *  have also removed the now unused variable 's'.  Subsequently, I have also
 *  eliminated the BSD function rindex() to allow this to run under SOLARIS,
 *  by using the following Rindex() function and using it instead. I have
 *  also changed the name of the symlink variable to symblink to prevent lint
 *  complaining about it hiding the name of a system function. Moreover, I
 *  have removed the TAB characters from the original code, since the TAB
 *  settings being used didn't match those in the rest of this file and so
 *  made the code layout very odd indeed.
 */

#ifdef IMPZ_PROTOTYPES_OK

char *Rindex(
   char *s,
   char c)

#else

char *Rindex(s, c)
   char *s, c;

#endif

{
   char *ptr = s;
   char *cptr = (char *) 0;
   while (*ptr) { if (*ptr == c) cptr = ptr; ptr++; }
   return (cptr);
}

/*  ----------------------------------------------------------------------- */

#include <errno.h>
#include <sys/param.h>

#ifndef   X_OK
#define   X_OK   1         /* for "access" */
#endif

#ifndef   MAXPATHLEN
#define   MAXPATHLEN   1024
#endif

#ifndef   ENAMETOOLONG
#define   ENAMETOOLONG   EINVAL
#endif

int                     findx ();     /* get location of directory */
static int              resolve ();   /* get link resolution name */
#ifdef   FINDX_TESTMAIN
/*
 * An example of how to use this routine.
 */
extern char            *getenv ();            /* read value from environment */
char                   *pn = (char *) 0;      /* program name */
char                   *rn = (char *) 0;      /* run name */
char                    rd[MAXPATHLEN];       /* run directory */
char                    wd[MAXPATHLEN] = "."; /* working directory */

int
main (argc, argv)
    int                     argc;
    char                  **argv;
{
    findx (*argv, wd, rd, &pn, &rn, getenv ("PATH"));
    printf ("%s: %s running in %s from %s\n", pn, rn, wd, rd);
    return 0;
}
#endif

int
findx (cmd, cwd, dir, pgm, run, path)
    char                   *cmd;
    char                   *cwd;
    char                   *dir;
    char                  **pgm;
    char                  **run;
    char                   *path;
{
    int                     rv = 0;
    char                   dstr[MAXPATHLEN],
                           *dp,
                           *f;
    if (!cmd || !*cmd || !cwd || !dir) {
       errno = EINVAL;        /* stupid arguments! */
       return -1;
    }
    if (!path || !*path)      /* missing or null path */
       path = ".";            /* assume sanity */

    if (*cwd != '/')
       if (!(getcwd (cwd, MAXPATHLEN)))
          return -1;          /* cant get working directory */

    f = Rindex (cmd, '/');
    if (pgm)                  /* user wants program name */
       *pgm = f ? f + 1 : cmd;

    if (dir) {                /* user wants program directory */
       rv = -1;
       if (*cmd == '/')       /* absname given */
          rv = resolve ("", cmd + 1, dir, run);
       else if (f)            /* relname given */
          rv = resolve (cwd, cmd, dir, run);
       else if ((f = path)) { /* from searchpath */
          rv = -1;
          errno = ENOENT;     /* errno gets this if path empty */
          while (*f && (rv < 0)) {
             dp = dstr;
             while (*f && (*f != ':'))
                *dp++ = *f++;
             if (*f) {
                f++;
                *dp = '\0';
             }
             if (dstr[0] == '/')
                rv = resolve (dstr, cmd, dir, run);
             else {
                char                    abuf[MAXPATHLEN];
                sprintf (abuf, "%s/%s", cwd, dstr);
                rv = resolve (abuf, cmd, dir, run);
             }
          }
       } 
    }
    return rv;
}
/*
 * resolve - check for specified file in specified directory. sets up dir,
 * following symlinks. returns zero for success, or -1 for error (with
 * errno set properly)
 */
static int
resolve (indir, cmd, dir, run)
   char                   *indir;   /* search directory */
   char                   *cmd;     /* search for name */
   char                   *dir;     /* directory buffer */
   char                  **run;     /* resultion name ptr ptr */
{
   char                   *p;
#ifdef   ELOOP
   int                     sll;
   char                    symblink[MAXPATHLEN + 1];
#endif

   errno = ENAMETOOLONG;
   if ((int) (strlen (indir) + strlen (cmd) + 2) > MAXPATHLEN)
      return -1;

   sprintf (dir, "%s/%s", indir, cmd);
   if (access (dir, X_OK) < 0)
      return -1;         /* not an executable program */

#ifdef   ELOOP
   while ((sll = readlink (dir, symblink, MAXPATHLEN)) >= 0) {
      symblink[sll] = 0;
      if (*symblink == '/')
         strcpy (dir, symblink);
      else
         sprintf (Rindex (dir, '/'), "/%s", symblink);
   }
   if (errno != EINVAL)
      return -1;
#endif

   p = Rindex (dir, '/');
   *p++ = '\0';
   if (run)              /* user wants resolution name */
      *run = p;
   return 0;
}
