#!/bin/sh
#\
exec wish  "$0" ${1+"$@"}
#
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ACMM:ozpozFpcol/ws/src/residuals.tcl,v 1.63 07-Oct-2008 11:10:26+10 tjf $"
#
#
# who       when      what
# --------  --------  ----------------------------------------------
# ssmedley  07/09/01  created
# tfarrell  21/08/02  Changed initial scales
# tfarrell  22/08/02  Chaned initial size and scroll area to more appropiate values.
# tfarrell  24/08/02  Revamp the way the residual output stuff works, based
#                     on Matthew Colless's work in the area.  Corresponding
#                     change to fpcolMdlCalFit.c
# tfarrell  24/08/02  Add print command (pretty basic).
# tfarrell  26/08/02  Add compass, scale and display original file name.
#                     Remove mean offset (as it confuses the rest of the
#                     stuff).
#
#


# This function stores the info about a fibre position & it's
# error in memory.
proc addRecord {n x y residualX residualY offsetX offsetY} {
	global _x _y _residualX _residualY _offsetX _offsetY

	set _x($n) $x
	set _y($n) $y
	set _residualX($n) $residualX
	set _residualY($n) $residualY
	set _offsetX($n) $offsetX
	set _offsetY($n) $offsetY

}

proc removeMeanOffset { } {
    global _offsetX _offsetY
    # First work out the total in each of X and Y and the number of values.
    set totalX 0
    set totalY 0
    set n 0
    foreach f [array names _offsetX] {
        incr n
        set totalX [expr $totalX + $_offsetX($f)]
        set totalY [expr $totalY + $_offsetY($f)]
    }
    puts "$n fibres, total offset $totalX, $totalY"

    if { $n == 0 } {
        return
    }
    # Get the mean.
    set meanX [expr $totalX/$n]
    set meanY [expr $totalY/$n]

    # Now remove this mean for each entry.
    puts "Removing mean offset of $meanX, $meanY"
    foreach f [array names _offsetX] {
        set  _offsetX($f) [expr $_offsetX($f) - $meanX]
        set  _offsetY($f) [expr $_offsetY($f) - $meanY]
    }

}

# a second command line argument of '-V' enables extra logging info.
set bVerbose 0
if {[llength $argv] > 1} {
	if {[string compare [lindex $argv 1] "-V"] == 0} {
		set bVerbose 1
	}
}

set filename $argv
# open the file containing the residuals.
if [catch {open [lindex $filename 0] r} fileId] {
	puts stderr "Error: $fileId"
	exit
}

set northX 406
set northY 404
set eastX  -404
set eastY  406
set CompassOriginX 370000
set CompassOriginY 370000

# load all data from file into memory.
foreach line [split [read $fileId] \n] {
	# ignore blank lines.
	if {[string length $line] == 0} {
		continue
	}
	# 
        # Special and optional header lines
        #
	if { [string match "#NAME=*" $line] } {
            set filename [string range $line 6 end]
            puts "Raster Name is  \"$filename\""
            continue
        } elseif { [string match "#COMPASS=*" $line] } {
            set the_rest  [string range $line 9 end]
            set n [scan $the_rest {%d %d %d %d} northX northY eastX eastY]
            if { $n != 4 }  {
                puts "Found invalid compass line \"$line\", \"$the_rest\""
            } 
            continue
	} elseif {[string match "#" [string index $line 0]]} {
            # All other comments.
            continue
        }
	set n [scan $line \
				{%d %f %f %f %f %f %f} \
				fibre \
				x \
				y \
				initialX \
				initialY \
				fittedX \
				fittedY]

	if {$n != 7} {
		puts "error: could not parse line:"
		puts "<$line>\nAborting."
		exit 1
	}
	if {$bVerbose == 1} {
		puts "fibre=$fibre\nxOld=$xOld\nxNew=$xNew\nxResidual=$xResidual"
		puts "yOld=$yOld\nyNew=$yNew\nyResidual=$yResidual"
	}
	addRecord $fibre \
			  $fittedX \
			  $fittedY \
			  [expr $x - $initialX] \
			  [expr $y - $initialY] \
			  [expr $x - $fittedX] \
			  [expr $y - $fittedY]
}

close $fileId

puts "loaded [array size _x] records"

proc max {a b} {
	return [expr {($a > $b) ? $a : $b}]
}

proc min {a b} {
	return [expr {($a < $b) ? $a : $b}]
}

# find the minimum bounding box encompassing all fibres.
set dataMinX  999999
set dataMinY  999999
set dataMaxX -999999
set dataMaxY -999999
# dataMaxOffset is the number of microns an error vector
# may protrude outside this bounding box.
set dataMaxOffset 0

foreach f [array names _x] {
	set dataMinX [min $dataMinX $_x($f)]
	set dataMaxX [max $dataMaxX $_x($f)]
	set dataMinY [min $dataMinY $_y($f)]
	set dataMaxY [max $dataMaxY $_y($f)]
	set dataMaxOffset [max $dataMaxOffset [expr abs($_residualX($f))]]
	set dataMaxOffset [max $dataMaxOffset [expr abs($_residualY($f))]]
}

# plate radius: ~445 millimetres
set plateRadius 445000
set bShowCH 1
set bShowPlate 1
set bAutoCentre 1
set bShowFibres 1
set bShowResiduals 1
set bShowCompass 1
set bShowOffsets 1
set defaultScale 2000
set defaultResidualMag 300
set defaultOffsetMag 300
# scale: microns per pixel
set scale $defaultScale
# residualMag: how much to exaggerate residual by.
set residualMag $defaultResidualMag
set offsetMag	$defaultOffsetMag


if {$bVerbose == 1} {
	puts "minX=$minX\nmaxX=$maxX\nminY=$minY\nmaxY=$maxY\nmaxOffset=$maxOffset"
}

wm title . "Residuals : $filename"
menu .menubar
. config -menu .menubar
foreach m {File Options Help} {
	set $m [menu .menubar.m$m -tearoff 0]
	.menubar add cascade -label $m -menu .menubar.m$m
}
$File add command -label Quit -command exit
$File add command -label Print -command PrintCanvas
$Options add command -label "Reset Mag/Scale" \
					 -command { set scale $defaultScale; \
								set residualMag $defaultResidualMag; \
								set offsetMag $defaultOffsetMag; \
								plot }
$Options add command -label Centre -command {centre}
$Options add check -label AutoCentre \
				   -variable bAutoCentre \
				   -command {
					   	if {$bAutoCentre == 1} {
							centre
						}
				   	}
$Options add check -label "Show Fibres" \
				   -variable bShowFibres \
				   -command { plot } -selectcolor blue  
$Options add check -label "Show Crosshair" \
				   -variable bShowCH \
				   -command { plot } -selectcolor black 
$Options add check -label "Show Plate" \
				   -variable bShowPlate \
				   -command { plot } -selectcolor black
$Options add check -label "Show Residuals" \
				   -variable bShowResiduals \
				   -command { plot } -selectcolor Blue
$Options add check -label "Show Offsets" \
				   -variable bShowOffsets \
				   -command { plot } -selectcolor Red
$Options add check -label "Show Compass" \
				   -variable bShowCompass \
				   -command { plot } -selectcolor Blue



# create a scrollable canvas
frame .f
pack .f -expand 1 -fill both
canvas .f.c -background white \
			-scrollregion {0 0 700 700} \
			-width 700 \
			-height 700 \
			-yscrollcommand [list .f.yScroll set] \
			-xscrollcommand [list .f.xScroll set] \
			-highlightthickness 1 \
			-highlightcolor blue \
			-highlightbackground blue \
                        -xscrollincrement 20 -yscrollincrement 20

# when moving the mouse around the canvas, display info	about
# any/all fibres under the cursor. Note that we must convert
# canvas VIEW coordinates to canvas coordinates before passing
# them to <showInfo>.
bind .f.c <Motion> { showInfo %W [%W canvasx %x] [%W canvasy %y] }

scrollbar .f.yScroll -orient vertical \
					 -command [list .f.c yview]
scrollbar .f.xScroll -orient horizontal \
					 -command [list .f.c xview]
grid .f.c .f.yScroll -sticky news
grid .f.xScroll .f.c -sticky news
grid rowconfigure .f 0 -weight 1
grid columnconfigure .f 0 -weight 1


label .lResMag -text "Residual Mag.: "
pack .lResMag -side left
entry .eResMag -textvariable residualMag \
			-relief sunken \
			-background white \
			-width 8
bind .eResMag <Return> { plot }
pack .eResMag -side left
label .lOffMag -text "Offset Mag.: "
pack .lOffMag -side left
entry .eOffMag -textvariable offsetMag \
			-relief sunken \
			-background white \
			-width 8
bind .eOffMag <Return> { plot }
pack .eOffMag -side left
label .lScale -text "Scale: "
pack .lScale -side left
entry .eScale -textvariable scale \
			  -relief sunken \
			  -background white \
			  -width 8
bind .eScale <Return> { plot }
pack .eScale -side left

catch {font create myfont -family Helvetica -size 16 -weight bold}

# Tk doesn't allow you to get the bounding box of some text
# without first putting the text on a canvas widget. This is a
# pain in the butt to say the least. To get around this
# deficiency we create a hidden canvas that we can use soley
# for calculating the bounding box of text placed on it.
canvas .hidden
proc bboxOfText {text} {
	global .hidden
	
	set t [.hidden create text 0 0 -text $text -anchor nw -font fixed]
	set bbox [.hidden bbox $t]
	.hidden delete all
	return $bbox
}

# show info about fibre/error under cursor.
proc showInfo {c x y} {
    global _x _y _residualX _residualY _offsetX _offsetY
    
    $c delete balloon
    # get any objects currently in the vicinity of the cursor.
    set ids [$c find overlapping [expr $x - 1] \
            [expr $y - 1] \
            [expr $x + 1] \
            [expr $y + 1]]
    # get the tags of each of these objects to find out if we're
    # really interested in them or not.
    set text ""
    foreach id $ids {
        set tags [$c gettags $id]
        foreach tag $tags {
            # looking for tags like: "fibre%d" & "error%d"
            if {[string compare "fibre" [string range $tag 0 4]] == 0} {
                set n [string range $tag 5 [string length $tag]]
                append text [format "%3d:%11.3f,%11.3f\n" $n $_x($n) $_y($n)]
            } elseif {[string compare "error" [string range $tag 0 4]] == 0} {
                set n [string range $tag 5 [string length $tag]]
                append text [format "%3d:%11.6f,%11.6f R\n" $n $_residualX($n) $_residualY($n)]
            } elseif {[string compare "offsetsScale"  $tag] == 0 } {
                append text "Offsets's scale, 1 arc-second"
            } elseif {[string compare "offset" [string range $tag 0 5]] == 0} {
                set n [string range $tag 6 [string length $tag]]
                append text [format "%3d:%11.6f,%11.6f O\n" $n $_offsetX($n) $_offsetY($n)]
            } elseif {[string compare "compass" $tag] == 0 } {
                append text "Compass\n  Each arrow is 150 arc-seconds long\n  on the plate scale"
            } elseif {[string compare "residScale" $tag] == 0 } {
                append text "Residual's scale, 1 arc-second"
            }
        }
    }
    
    if {[string length $text] > 0} {
        # chop off newline.
        set text [string trimright $text]
        set bbox [bboxOfText $text]
        $c create rect [expr $x - 1] \
                [expr $y + 1] \
                [expr $x - 1 - [lindex $bbox 2]] \
                [expr $y + 1 + [lindex $bbox 3]] \
                -fill yellow \
                -outline black \
                -tag balloon
        $c create text [expr $x - 1] \
                [expr $y + 1] \
                -text $text \
                -fill black \
                -anchor ne \
                -font fixed \
                -tag balloon
    }
} 
global PrintCommand PrintColour
set PrintCommand  "lp -d laser1"
set PrintColour 1

proc PrintCanvas { } {
    
    set win ".printCommand"
    set NotCreated [catch "wm state $win" state]
    if {$NotCreated} {
        toplevel $win 
        wm title $win "Residuals Print Command "
        wm iconname $win ResidPrint
        wm minsize  $win 5 5
        wm maxsize  $win 1000 800
       
        frame $win.f1
        frame $win.f2

        pack $win.f1 $win.f2 -side top -fill both -expand 1

        label $win.f1.label -text "Enter Print Command:"
        entry $win.f1.entry -textvariable PrintCommand
        checkbutton $win.f1.colour -text "Colour" -variable PrintColour 

        pack $win.f1.label $win.f1.entry $win.f1.colour -side left -fill x -expand 1

        button $win.f2.print -text "Print" -command "PrintIt ; wm withdraw $win"
        button $win.f2.close -text "Close" -command "wm withdraw $win"
        
        pack $win.f2.print $win.f2.close -side left -fill x -expand 1


    } else {
        wm deiconify $win
        raise $win
    }
    

            
}

proc PrintIt { } {

    global PrintCommand PrintColour
#    puts "PrintCommand = $PrintCommand, colour = $PrintColour"

    set outname "/tmp/resid.ps"

    if $PrintColour {
        set cmode "color"
    } else {
        set cmode "gray"
    }

    if [file exists $outname] {
        file delete -force $outname
    }

    DrawPrintBits

    set result [.f.c postscript -colormode $cmode -file $outname -pagewidth 180m]

    eval exec  $PrintCommand $outname


    if [file exists $outname] {
        file delete -force $outname
    }   

    .f.c delete PrintBits
}

proc DrawPrintBits { } {
    global bShowPlate dataMaxX dataMaxY dataMinX dataMinY plateRadius

    
    # the bounding box encompasses all fibres but only the plate
    # if we are actually displaying it.
    set maxX [expr {($bShowPlate == 1) ? [max $dataMaxX $plateRadius] : $dataMaxX}]
    set maxY [expr {($bShowPlate == 1) ? [max $dataMaxY $plateRadius] : $dataMaxY}]
    set minX [expr {($bShowPlate == 1) ? [min $dataMinX -$plateRadius] : $dataMinX}]
    set minY [expr {($bShowPlate == 1) ? [min $dataMinY -$plateRadius] : $dataMinY}]
    
#    puts "$maxX $maxY $minX $minY"

#    puts "xview = [.f.c xview]"
#    puts "yview = [.f.c yview]"

    .f.c xview moveto 0
    .f.c yview moveto 0

#    puts "xview = [.f.c xview]"
#    puts "yview = [.f.c yview]"

   # determine canvas width & height.
    set canvasWidth  [.f.c cget -width]
    set canvasHeight [.f.c cget -height]

    set boxX2 [expr $canvasWidth - 1]
    set boxY2 [expr $canvasHeight - 1]

#    puts "$canvasWidth $canvasHeight"

    .f.c create rectangle 1 1 $boxX2 $boxY2 -tags PrintBits -width 2


}

# centre (0,0) on the canvas view.
proc centre { } {
	global .f.c canvasWidth canvasHeight bVerbose

	set c [posToCanvas 0 0]

	# <viewX>: number of pixels in X direction of canvas <.f.c>
	# visible on screen.
	# <viewY>: number of pixels in Y direction of canvas <.f.c>
	# visible on screen.
	# puts -nonewline "waiting for visibility ... "
	# tkwait visibility .f.c
	# puts "got it"
	update idletasks
	set viewX [winfo width .f.c]
	set viewY [winfo height .f.c]

	# moveto command wants to know fraction that is to
	# appear offscreen.
	set left [expr [lindex $c 0] - ($viewX / 2)]
	set top [expr [lindex $c 1] - ($viewY / 2)]
	set width [max $canvasWidth $viewX]
	set height [max $canvasHeight $viewY]

	if {$bVerbose == 1} {
		puts "c is: $c"
		puts "vX=$viewX vY=$viewY"
		puts "l=$left t=$top"
		puts "width=$width height=$height"
	}

	.f.c xview moveto [expr $left / $width]
	.f.c yview moveto [expr $top / $height]
}

# posToCanvas: x/y (microns) to canvas coordinates.
proc posToCanvas {x y} {
    global canvasPadX canvasPadY minX minY maxX maxY scale
     

    set canvasX [expr $canvasPadX + (($x - $minX) / $scale)]
    set canvasY [expr $canvasPadY + (($y + $minY) / $scale * -1)]
    return [list $canvasX $canvasY]
}

# display fibre positions & vectors denoting the
# (exaggerated) error.
proc plot {} {
	global minX minY maxX maxY scale residualMag offsetMag plateRadius
	global _x _y _residualX _residualY _offsetX _offsetY canvasWidth canvasHeight
	global canvasPadX canvasPadY padding maxOffset bVerbose bAutoCentre
	global dataMaxX dataMaxY dataMinX dataMinY dataMaxOffset
	global bShowOffsets bShowPlate bShowCH bShowResiduals bShowFibres bShowCompass

	# the bounding box encompasses all fibres but only the plate
	# if we are actually displaying it.
	set maxX [expr {($bShowPlate == 1) ? [max $dataMaxX $plateRadius] : $dataMaxX}]
	set maxY [expr {($bShowPlate == 1) ? [max $dataMaxY $plateRadius] : $dataMaxY}]
	set minX [expr {($bShowPlate == 1) ? [min $dataMinX -$plateRadius] : $dataMinX}]
	set minY [expr {($bShowPlate == 1) ? [min $dataMinY -$plateRadius] : $dataMinY}]
	set maxOffset $dataMaxOffset
	
	# padding is the buffer (canvas space) around the (imaginary)
	# bounding box that encompasses all fibres. It is needed so that
	# offset arrows are visible when they protrude outside of this
	# bounding box.
	set padding [expr ($maxOffset * [max $residualMag $offsetMag]) / $scale]
	set canvasPadX $padding
	set canvasPadY $canvasPadX

	# determine canvas width & height.
	set canvasWidth [expr (2 * $canvasPadX) + (($maxX - $minX) / $scale)]
	set canvasHeight [expr (2 * $canvasPadY) + (($maxY - $minY) / $scale)]
    
        .f.c config -width $canvasWidth -height $canvasHeight

	.f.c config -scrollregion [list 0 0 $canvasWidth $canvasHeight]
	if {$bVerbose == 1} {
		puts "scrollregion=($canvasWidth,$canvasHeight)"
	}

	# clean the canvas before we draw anything on it.
	.f.c delete all

	# draw crosshair
	if {$bShowCH == 1} {
		set crosshairColor black
		set from [posToCanvas $minX 0]
		set to [posToCanvas $maxX 0 ]
		.f.c create line [lindex $from 0] \
					   [lindex $from 1] \
					   [lindex $to 0] \
					   [lindex $to 1] \
					   -fill $crosshairColor \
					   -tag crosshair
		set from [posToCanvas 0 $minY]
		set to [posToCanvas 0 $maxY]
		.f.c create line [lindex $from 0] \
					   [lindex $from 1] \
					   [lindex $to 0] \
					   [lindex $to 1] \
					   -fill $crosshairColor \
					   -tag crosshair
	}

	# draw plate outline
	if {$bShowPlate == 1} {
		set from [posToCanvas -$plateRadius -$plateRadius]
		set to [posToCanvas $plateRadius $plateRadius]
		.f.c create oval [lindex $from 0] \
						 [lindex $from 1] \
						 [lindex $to 0] \
						 [lindex $to 1] \
						 -outline black \
						 -tag plateBoundary
	}

	# radius of fibre button is actually <$radius> + 1/2 (in pixels).
	set radius 2
	set buttonColor blue

	foreach f [array names _x] {
		set coord [posToCanvas $_x($f) $_y($f)]

		if {$bShowFibres == 1} {
			# Draw the actual fibre
			.f.c create rect [expr [lindex $coord 0] - $radius] \
                                [expr [lindex $coord 1] - $radius] \
                                [expr [lindex $coord 0] + $radius] \
                                [expr [lindex $coord 1] + $radius] \
                                -fill $buttonColor \
                                -tag "fibre$f"
		}
		if {$bShowResiduals == 1} {
			# Draw the (exaggerated) residual arrow.
			set errorCoord [posToCanvas [expr $_x($f) + \
                                ($_residualX($f) * \
                                $residualMag)] \
                                [expr $_y($f) + \
                                ($_residualY($f) * \
                                $residualMag)]]
			.f.c create line [lindex $coord 0] \
                                [lindex $coord 1] \
                                [lindex $errorCoord 0] \
                                [lindex $errorCoord 1] \
                                -arrow last -fill Blue\
                                -tag "error$f"
		}
		if {$bShowOffsets == 1} {
                    # draw the offset arrow.
                    set offsetCoord [posToCanvas [expr $_x($f) + \
                            ($_offsetX($f) * \
                            $offsetMag)] \
                            [expr $_y($f) + \
                            ($_offsetY($f) * \
                            $offsetMag)]]
                    .f.c create line [lindex $coord 0] \
                            [lindex $coord 1] \
                            [lindex $offsetCoord 0] \
                            [lindex $offsetCoord 1] \
                            -arrow last \
                            -fill Red \
                            -tag "offset$f"
		}
	}
        # Scales for residuals and offsets.
        if {$bShowResiduals } {
            # We want to show the residuals scale.  This is drawn on the opposite side
            # of the plate from the compass (below).
            global  CompassOriginX CompassOriginY
            set xStart [expr ($CompassOriginX * -1) - 100000]
            set yStart [expr ($CompassOriginY * -1) - 100000]
            
            # The length should be one arc-second.  This can be calculated from
            # the compass vectors - which are one arc-second long
            global northX northY
            set length [expr sqrt($northX*$northX + $northY*$northY)]

            set originCoord [posToCanvas $xStart $yStart]


            set errorCoord [posToCanvas \
                    [expr $xStart + ($length * $residualMag)] \
                    $yStart]

            # Draw the (exaggerated) residual arrow.
            .f.c create line [lindex $originCoord 0] \
                    [lindex $originCoord 1] \
                    [lindex $errorCoord 0] \
                    [lindex $errorCoord 1] \
                    -arrow last -fill Blue\
                    -tag "residScale"
  
            .f.c create text  \
                    [lindex $originCoord 0] \
                    [expr [lindex $originCoord 1]-10] -anchor w \
                    -text "Residuals (1 arcsec)"  -tag "residScale"

        }
        if {$bShowOffsets } {
            # We want to show the offset scale.  This is drawn on the opposite side
            # of the plate from the compass (below) and slightly below the residual scale (above)
            global  CompassOriginX CompassOriginY
            set xStart [expr ($CompassOriginX * -1) - 100000]
            set yStart [expr ($CompassOriginY * -1) - 160000]
            
            # The length should be one arc-second.  This can be calculated from
            # the compass vectors - which are one arc-second long
            global northX northY
            set length [expr sqrt($northX*$northX + $northY*$northY)]

            set originCoord [posToCanvas $xStart $yStart]


            set errorCoord [posToCanvas \
                    [expr $xStart + ($length * $offsetMag)] \
                    $yStart]

            # Draw the (exaggerated) offset arrow.
            .f.c create line [lindex $originCoord 0] \
                    [lindex $originCoord 1] \
                    [lindex $errorCoord 0] \
                    [lindex $errorCoord 1] \
                    -arrow last -fill Red \
                    -tag "offsetsScale"
  
            .f.c create text  \
                   [lindex $originCoord 0] \
                    [expr [lindex $originCoord 1]-10] -anchor w \
                    -text "Offsets (1 arcsec)"  -tag "offsetsScale"


        }
        if {$bShowCompass == 1 } {
            # The compass is drawn from 370000, 370000 - outside the circle.
            # The variables below givethe north and east vectors for 1 arc-second.
            #   We show a vector of length 1 arc-minute.

            global northX northY eastX eastY CompassOriginX CompassOriginY

            set originCoord [posToCanvas $CompassOriginX $CompassOriginY]

            # Drawing the north vector.
            set compassNorthCoord [posToCanvas \
                    [expr $CompassOriginX + ($northX*150)] \
                    [expr $CompassOriginY + ($northY*150)]] 

            .f.c create line [lindex $originCoord 0] \
                    [lindex $originCoord 1] \
                    [lindex $compassNorthCoord 0] \
                    [lindex $compassNorthCoord 1] \
                    -arrow last \
                    -fill Blue \
                    -tag "compass" 

            .f.c create text  \
                    [expr [lindex $compassNorthCoord 0] + 10] \
                    [expr [lindex $compassNorthCoord 1] + 10] \
                    -text "N"  -tag compass
                    

            # Drawing the east vector.
            set compassEastCoord [posToCanvas \
                    [expr $CompassOriginX + ($eastX*150)] \
                    [expr $CompassOriginY + ($eastY*150)]] 


            .f.c create line [lindex $originCoord 0] \
                    [lindex $originCoord 1] \
                    [lindex $compassEastCoord 0] \
                    [lindex $compassEastCoord 1] \
                    -arrow last \
                    -fill Blue \
                    -tag "compass"           

            .f.c create text  \
                    [expr [lindex $compassEastCoord 0] - 10] \
                    [expr [lindex $compassEastCoord 1] + 10] \
                    -text "E"  -tag compass
  


        }
	if {$bAutoCentre == 1} {
		#centre
	}
        global filename
        .f.c create text [expr $canvasWidth/2] 20 -anchor center  \
                -text "$filename" -font myfont

}
removeMeanOffset
plot

