/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/test/fpfitTest.c,v 1.63 07-Oct-2008 11:10:29+10 tjf $
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* jab       07/05/01  created/
* tjf       10/07/01  Added VLT style header.
* tjf       11/07/01  Rename OzPos to OzPoz.
*/

/************************************************************************
*   NAME
*       fpfitTest - A program to test the least squares fitting routine OzPozFit
*
*   SYNOPSIS
*       fpfitTest
*
*   DESCRIPTION
*       
*
***
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
*------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fpcolMdl.h"


int main()

{
    StatusType status = STATUS__OK;
    FpilModelType *model;
    FpilModelType *model2;
    double pars[7];
    double params[2];
    int pfit[7];
    double ra[9],dec[9];
    double ora[9],odec[9];
    double cra,cdec;
    double mjd;
    double x,y;
    double ra2,dec2;
    double xp[9],yp[9];
    int i;
    double rms,psd;
    double pars2[7];
    double pvar[7];
    int rd;
    double r;
/*  set up a model  -  this is the initial (not quite correct) model 
    representing that used to configure the fibres   */
            
    pars[0] = 0.0;
    pars[1] = 1.0;
    pars[2] = 0.0;
    pars[3] = 0.0;
    pars[4] = 0.0;
    pars[5] = 1.0;
    pars[6] = 100.0;

    fpcolMdlCreateModel(7,pars,1,&model,&status);
    printf("fpcolMdlCreateModel - status = %ld \n",(long)status);
    
    mjd = 51910.17;   /*  Jan 1 2001 - approx local midnight  */
    
    fpcolMdlCvtInit(model,mjd,0.0,273.0,700.0,0.5,0.55,params,&status);
    printf("fpcolMdlCvtInit - status = %ld \n",(long)status);
    
/*  Set up a grid of positions  */

    cra = 1.5;
    cdec = 0.0;
    
    ra[0] = 1.5;                     /* Centre */
    dec[0] = 0.0;
    
    ra[1] = 1.5 + 900/206265.0;      /* 900 arc sec E  */
    dec[1] = 0.0;
    
    ra[2] = 1.5 - 900/206265.0;      /* 900 arc sec W  */    
    dec[2] = 0.0;
    
    ra[3] = 1.5;                     /* 900 arc sec N  */
    dec[3] = 900/206265.0;
    
    ra[4] = 1.5;                     /* 900 arc sec S  */
    dec[4] = -900/206265.0;
    
    ra[5] = 1.5 + 900/206265.0;      /*  E and N  */
    dec[5] = 900/206265.0;
    
    ra[6] = 1.5 + 900/206265.0;      /*  E and S  */
    dec[6] = -900/206265.0;
    
    ra[7] = 1.5 - 900/206265.0;      /*  W and N  */
    dec[7] = 900/206265.0;
    
    ra[8] = 1.5 - 900/206265.0;      /*  W and S  */
    dec[8] = -900/206265.0;
    
/*  Transform the positions to xy using the initial model - this
    gives the xy values the fibres would be set at  */

    for (i=0;i<9;i++)
    {
        fpcolMdlRd2Xy(model,cra,cdec,ra[i],dec[i],mjd,&x,&y,&status);
        fpcolMdlXy2Pos(model,x,y,&(xp[i]),&(yp[i]),&status);
    }        
    
/*  Create a new model with different paramaters - this model represents
    the true model which we are trying to determine  */

    pars[0] = 100.5;
    pars[1] = 0.9999;
    pars[2] = 0.00005;
    pars[3] = 0.7;
    pars[4] = -0.00005;
    pars[5] = 0.9999;
    pars[6] = 130.0;

/*  set up the model  */
    
    fpcolMdlCreateModel(7,pars,1,&model2,&status);
    printf("fpcolMdlCreateModel - status = %ld \n",(long)status);
    
    mjd = 51910.17;   /*  Jan 1 2001 - approx local midnight  */
    
    fpcolMdlCvtInit(model2,mjd,0.0,273.0,700.0,0.5,0.55,params,&status);
    printf("fpcolMdlCvtInit - status = %ld \n",(long)status);
    
/*  Convert xy back to RA, Dec with new model and note offset  */ 

/*  The positions ra2 and dec2 are the positions the fibres will actually
    be set up at. To center the stars on the fibres we would need to
    move the telescope to the correct position ra[i], dec[i]. The offsets 
    (ra[i] - ra2 and dec[i] - dec2) are the offset values which have to be 
    given to the fitting program   */
    
/*  Add random noise of range up to +/- 0.1 arc sec  */       
    
    srand(1234);
    for (i=0;i<9;i++)
    {
        fpcolMdlPos2Xy(model2,xp[i],yp[i],&x,&y,&status);
        fpcolMdlXy2Rd(model2,cra,cdec,x,y,mjd,&ra2,&dec2,&status);
        ora[i] = ra[i] - ra2;
        rd = rand();
        r = (double)rd/(double)(RAND_MAX);
   /*     ora[i] = ora[i] + 0.2/206265.0 * (r-0.5); */
        odec[i] = dec[i] - dec2;
        rd = rand();
        r = (double)rd/(double)(RAND_MAX);
    /*    odec[i] = odec[i] + 0.2/206265.0 * (r-0.5); */
        
    }        
        
/*  Do the fit - we fit all parameters here. To fix one of the parameters
    set the corresponding element of pfit to 1  */

    pfit[0] = 0;
    pfit[1] = 0;
    pfit[2] = 0;
    pfit[3] = 0;
    pfit[4] = 0;
    pfit[5] = 0;
    pfit[6] = 0;
    
    fpcolMdlFit(9,model,mjd,cra,cdec,ra,dec,ora,odec,pfit,pvar,&rms,&psd,&status);
    printf("Model fit status = %ld \n",(long)status);
    
/*  report the rms and psd of the fit  */    
    
    printf("rms = %10.5f  psd = %10.5f \n",rms,psd);
    
/*  "model" now contains the newly fitted model - get the parameters
    and report them - they should be very close to the values originally
    set for model2   */    
    
    fpcolMdlGetModel(model,7,pars2,&status);
    
    for (i=0; i<7; i++)
    {
        printf ("%12.8f +/- %12.8f\n",pars2[i],sqrt(pvar[i]));
    }
    return 0;
}        
    
        
