/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: ACMM:ozpozFpcol/ws/test/fpmdlTest.c,v 1.63 07-Oct-2008 11:10:27+10 tjf $
*
*
* who       when      what
* --------  --------  ----------------------------------------------
* jab       ?/?/01  created/
* tjf       10/07/01  Added VLT style header.
*/

/************************************************************************
*   NAME
*       fpmdlTest - A program to test the FLAMES/FP (OzPoz) telescope model.
*
*   SYNOPSIS
*       fpmdlTest
*
*   DESCRIPTION
*       
*
***
*
*   CAUTIONS
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
*
*------------------------------------------------------------------------
*/

#include <stdio.h>
#include "slamac.h"
#include "fpcolMdl.h"

void Report(void* clientData, char* str, StatusType *status)
{
    printf("%s\n",str);
}    


int main()

{
    FpilModelType *model;
    FpilModelType *model2;
    double pars[7];
    double params[2];
    StatusType status = STATUS__OK;
    double mjd;
    double cra,cdec;
    double ra,dec;
    double x,y;
    double xp,yp;
    double ra2,dec2;
    
    pars[0] = 0.0;
    pars[1] = 1.0;
    pars[2] = 0.0;
    pars[3] = 0.0;
    pars[4] = 0.0;
    pars[5] = 1.0;
    pars[6] = 100.0; 
    
    params[0] = DPI;

    printf("Calling CreateModel\n");
    fpcolMdlCreateModel(7,pars,1,&model,&status);
    printf("fpcolMdlCreateModel - status = %d \n",status);
    
    mjd = 51910.17;   /*  Jan 1 2001 - approx local midnight  */
    
    fpcolMdlCvtInit(model,mjd,0.0,273.0,700.0,0.5,0.55,params,&status);
    printf("fpcolMdlCvtInit - status = %d \n",status);
    
    cra = 1.5;
    cdec = 0.0;
    
    ra = 1.5 + 600.0/206265.0;     /*   600 arc sec east  */
    dec = 0.0;

    printf("Field centre - ra = %10.7f dec = %10.7f  status = %d\n",cra,cdec,status);

    printf("Source - ra = %10.7f dec = %10.7f  status = %d\n",ra,dec,status);

    
    fpcolMdlRd2Xy(model,cra,cdec,ra,dec,mjd,&x,&y,&status);
    printf("fpcolMdlRd2Xy - x = %8.1f y = %8.1f   status = %d\n",x,y,status);
    
    fpcolMdlXy2Pos(model,x,y,&xp,&yp,&status);
    printf("fpcolMdlXy2Pos - x = %8.1f y = %8.1f   status = %d\n",xp,yp,status);

    fpcolMdlPos2Xy(model,xp,yp,&x,&y,&status);
    printf("fpcolMdlPos2Xy - x = %8.1f y = %8.1f   status = %d\n",x,y,status);
    
    fpcolMdlXy2Rd(model,cra,cdec,x,y,mjd,&ra2,&dec2,&status);
    printf("fpcolMdlXy2Rd - ra = %10.7f dec = %10.7f  status = %d\n",ra2,dec2,status);
    

    fpcolMdlWriteModel(model,0,&status);
    printf("fpcolMdlWritemodel - status = %d\n",status);
    
    fpcolMdlReadModel(0,0,(FpilRptRoutineType)Report,0,1,&model2,&status);
    printf("fpcolMdlTransAccess - status = %d\n",status);

    fpcolMdlCvtInit(model2,mjd,0.0,273.0,700.0,0.5,0.55,params,&status);
    printf("fpcolMdlCvtInit - status = %d \n",status);

    fpcolMdlRd2Xy(model2,cra,cdec,ra,dec,mjd,&x,&y,&status);
    printf("fpcolMdlRd2Xy - x = %8.1f y = %8.1f   status = %d\n",x,y,status);
    
    fpcolMdlXy2Pos(model2,x,y,&xp,&yp,&status);
    printf("fpcolMdlXy2Pos - x = %8.1f y = %8.1f   status = %d\n",xp,yp,status);

    fpcolMdlPos2Xy(model2,xp,yp,&x,&y,&status);
    printf("fpcolMdlPos2Xy - x = %8.1f y = %8.1f   status = %d\n",x,y,status);
    
    fpcolMdlXy2Rd(model2,cra,cdec,x,y,mjd,&ra2,&dec2,&status);
    printf("fpcolMdlXy2Rd - ra = %10.7f dec = %10.7f  status = %d\n",ra2,dec2,status);
    
    return 0;
}    
    
    
   
